# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './mw4/gui/widgets/analyse.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AnalyseDialog(object):
    def setupUi(self, AnalyseDialog):
        AnalyseDialog.setObjectName("AnalyseDialog")
        AnalyseDialog.resize(1200, 800)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(AnalyseDialog.sizePolicy().hasHeightForWidth())
        AnalyseDialog.setSizePolicy(sizePolicy)
        AnalyseDialog.setMinimumSize(QtCore.QSize(1200, 800))
        AnalyseDialog.setMaximumSize(QtCore.QSize(2000, 1400))
        AnalyseDialog.setSizeIncrement(QtCore.QSize(10, 10))
        AnalyseDialog.setBaseSize(QtCore.QSize(10, 10))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        AnalyseDialog.setFont(font)
        self.verticalLayout = QtWidgets.QVBoxLayout(AnalyseDialog)
        self.verticalLayout.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.frame = QtWidgets.QFrame(AnalyseDialog)
        self.frame.setMinimumSize(QtCore.QSize(0, 90))
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.version = QtWidgets.QLineEdit(self.frame)
        self.version.setEnabled(False)
        self.version.setGeometry(QtCore.QRect(760, 65, 66, 25))
        self.version.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.version.setFont(font)
        self.version.setMouseTracking(False)
        self.version.setAcceptDrops(False)
        self.version.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.version.setText("")
        self.version.setReadOnly(True)
        self.version.setObjectName("version")
        self.filename = QtWidgets.QLineEdit(self.frame)
        self.filename.setEnabled(False)
        self.filename.setGeometry(QtCore.QRect(91, 5, 246, 25))
        self.filename.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.filename.setFont(font)
        self.filename.setMouseTracking(False)
        self.filename.setAcceptDrops(False)
        self.filename.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.filename.setText("")
        self.filename.setReadOnly(True)
        self.filename.setObjectName("filename")
        self.subframe = QtWidgets.QLineEdit(self.frame)
        self.subframe.setEnabled(False)
        self.subframe.setGeometry(QtCore.QRect(600, 35, 36, 25))
        self.subframe.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.subframe.setFont(font)
        self.subframe.setMouseTracking(False)
        self.subframe.setAcceptDrops(False)
        self.subframe.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.subframe.setText("")
        self.subframe.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.subframe.setReadOnly(True)
        self.subframe.setObjectName("subframe")
        self.label_20 = QtWidgets.QLabel(self.frame)
        self.label_20.setGeometry(QtCore.QRect(640, 65, 24, 25))
        self.label_20.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_20.setFont(font)
        self.label_20.setObjectName("label_20")
        self.label_21 = QtWidgets.QLabel(self.frame)
        self.label_21.setGeometry(QtCore.QRect(830, 65, 16, 25))
        self.label_21.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_21.setFont(font)
        self.label_21.setObjectName("label_21")
        self.label_13 = QtWidgets.QLabel(self.frame)
        self.label_13.setGeometry(QtCore.QRect(340, 65, 41, 25))
        self.label_13.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_13.setFont(font)
        self.label_13.setObjectName("label_13")
        self.modelOrthoError = QtWidgets.QLineEdit(self.frame)
        self.modelOrthoError.setEnabled(False)
        self.modelOrthoError.setGeometry(QtCore.QRect(270, 65, 66, 25))
        self.modelOrthoError.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.modelOrthoError.setFont(font)
        self.modelOrthoError.setMouseTracking(False)
        self.modelOrthoError.setAcceptDrops(False)
        self.modelOrthoError.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.modelOrthoError.setText("")
        self.modelOrthoError.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.modelOrthoError.setReadOnly(True)
        self.modelOrthoError.setObjectName("modelOrthoError")
        self.label_18 = QtWidgets.QLabel(self.frame)
        self.label_18.setGeometry(QtCore.QRect(405, 5, 42, 25))
        self.label_18.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_18.setFont(font)
        self.label_18.setObjectName("label_18")
        self.pointsWest = QtWidgets.QLabel(self.frame)
        self.pointsWest.setGeometry(QtCore.QRect(1045, 5, 91, 25))
        self.pointsWest.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.pointsWest.setFont(font)
        self.pointsWest.setObjectName("pointsWest")
        self.pointsEast = QtWidgets.QLabel(self.frame)
        self.pointsEast.setGeometry(QtCore.QRect(1045, 35, 91, 25))
        self.pointsEast.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.pointsEast.setFont(font)
        self.pointsEast.setObjectName("pointsEast")
        self.label_5 = QtWidgets.QLabel(self.frame)
        self.label_5.setGeometry(QtCore.QRect(540, 35, 56, 25))
        self.label_5.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.label_3 = QtWidgets.QLabel(self.frame)
        self.label_3.setGeometry(QtCore.QRect(405, 65, 49, 25))
        self.label_3.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.firmware = QtWidgets.QLineEdit(self.frame)
        self.firmware.setEnabled(False)
        self.firmware.setGeometry(QtCore.QRect(840, 65, 66, 25))
        self.firmware.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.firmware.setFont(font)
        self.firmware.setMouseTracking(False)
        self.firmware.setAcceptDrops(False)
        self.firmware.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.firmware.setText("")
        self.firmware.setReadOnly(True)
        self.firmware.setObjectName("firmware")
        self.label_10 = QtWidgets.QLabel(self.frame)
        self.label_10.setGeometry(QtCore.QRect(200, 65, 70, 25))
        self.label_10.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_10.setFont(font)
        self.label_10.setObjectName("label_10")
        self.time = QtWidgets.QLineEdit(self.frame)
        self.time.setEnabled(False)
        self.time.setGeometry(QtCore.QRect(760, 5, 146, 25))
        self.time.setMinimumSize(QtCore.QSize(25, 0))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.time.setFont(font)
        self.time.setMouseTracking(False)
        self.time.setAcceptDrops(False)
        self.time.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.time.setText("")
        self.time.setReadOnly(True)
        self.time.setObjectName("time")
        self.label_4 = QtWidgets.QLabel(self.frame)
        self.label_4.setGeometry(QtCore.QRect(685, 5, 70, 25))
        self.label_4.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.label_9 = QtWidgets.QLabel(self.frame)
        self.label_9.setGeometry(QtCore.QRect(200, 35, 69, 25))
        self.label_9.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_9.setFont(font)
        self.label_9.setObjectName("label_9")
        self.label_7 = QtWidgets.QLabel(self.frame)
        self.label_7.setGeometry(QtCore.QRect(910, 35, 54, 25))
        self.label_7.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_7.setFont(font)
        self.label_7.setObjectName("label_7")
        self.label_6 = QtWidgets.QLabel(self.frame)
        self.label_6.setGeometry(QtCore.QRect(685, 65, 66, 25))
        self.label_6.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.load = QtWidgets.QPushButton(self.frame)
        self.load.setGeometry(QtCore.QRect(5, 5, 80, 25))
        self.load.setMinimumSize(QtCore.QSize(80, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.load.setFont(font)
        self.load.setStyleSheet("")
        self.load.setObjectName("load")
        self.eposure = QtWidgets.QLineEdit(self.frame)
        self.eposure.setEnabled(False)
        self.eposure.setGeometry(QtCore.QRect(465, 35, 46, 25))
        self.eposure.setMinimumSize(QtCore.QSize(25, 0))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.eposure.setFont(font)
        self.eposure.setMouseTracking(False)
        self.eposure.setAcceptDrops(False)
        self.eposure.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.eposure.setText("")
        self.eposure.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.eposure.setReadOnly(True)
        self.eposure.setObjectName("eposure")
        self.label_19 = QtWidgets.QLabel(self.frame)
        self.label_19.setGeometry(QtCore.QRect(520, 65, 81, 25))
        self.label_19.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_19.setFont(font)
        self.label_19.setObjectName("label_19")
        self.label_2 = QtWidgets.QLabel(self.frame)
        self.label_2.setGeometry(QtCore.QRect(685, 35, 41, 25))
        self.label_2.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.label_16 = QtWidgets.QLabel(self.frame)
        self.label_16.setGeometry(QtCore.QRect(515, 35, 16, 25))
        self.label_16.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_16.setFont(font)
        self.label_16.setObjectName("label_16")
        self.winsorizedLimit = QtWidgets.QCheckBox(self.frame)
        self.winsorizedLimit.setGeometry(QtCore.QRect(1045, 65, 86, 25))
        self.winsorizedLimit.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.winsorizedLimit.setFont(font)
        self.winsorizedLimit.setObjectName("winsorizedLimit")
        self.modelTerms = QtWidgets.QLineEdit(self.frame)
        self.modelTerms.setEnabled(False)
        self.modelTerms.setGeometry(QtCore.QRect(90, 65, 46, 25))
        self.modelTerms.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.modelTerms.setFont(font)
        self.modelTerms.setMouseTracking(False)
        self.modelTerms.setAcceptDrops(False)
        self.modelTerms.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.modelTerms.setText("")
        self.modelTerms.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.modelTerms.setReadOnly(True)
        self.modelTerms.setObjectName("modelTerms")
        self.binning = QtWidgets.QLineEdit(self.frame)
        self.binning.setEnabled(False)
        self.binning.setGeometry(QtCore.QRect(465, 65, 46, 25))
        self.binning.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.binning.setFont(font)
        self.binning.setMouseTracking(False)
        self.binning.setAcceptDrops(False)
        self.binning.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.binning.setText("")
        self.binning.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.binning.setReadOnly(True)
        self.binning.setObjectName("binning")
        self.label_12 = QtWidgets.QLabel(self.frame)
        self.label_12.setGeometry(QtCore.QRect(340, 35, 41, 25))
        self.label_12.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_12.setFont(font)
        self.label_12.setObjectName("label_12")
        self.focalLength = QtWidgets.QLineEdit(self.frame)
        self.focalLength.setEnabled(False)
        self.focalLength.setGeometry(QtCore.QRect(600, 65, 36, 25))
        self.focalLength.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.focalLength.setFont(font)
        self.focalLength.setMouseTracking(False)
        self.focalLength.setAcceptDrops(False)
        self.focalLength.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.focalLength.setText("")
        self.focalLength.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.focalLength.setReadOnly(True)
        self.focalLength.setObjectName("focalLength")
        self.label_8 = QtWidgets.QLabel(self.frame)
        self.label_8.setGeometry(QtCore.QRect(5, 65, 41, 25))
        self.label_8.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_8.setFont(font)
        self.label_8.setObjectName("label_8")
        self.mirrored = QtWidgets.QLineEdit(self.frame)
        self.mirrored.setEnabled(False)
        self.mirrored.setGeometry(QtCore.QRect(965, 35, 61, 25))
        self.mirrored.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.mirrored.setFont(font)
        self.mirrored.setMouseTracking(False)
        self.mirrored.setAcceptDrops(False)
        self.mirrored.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.mirrored.setText("")
        self.mirrored.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.mirrored.setReadOnly(True)
        self.mirrored.setObjectName("mirrored")
        self.label = QtWidgets.QLabel(self.frame)
        self.label.setGeometry(QtCore.QRect(405, 35, 59, 25))
        self.label.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.label_14 = QtWidgets.QLabel(self.frame)
        self.label_14.setGeometry(QtCore.QRect(140, 35, 41, 25))
        self.label_14.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_14.setFont(font)
        self.label_14.setObjectName("label_14")
        self.label_15 = QtWidgets.QLabel(self.frame)
        self.label_15.setGeometry(QtCore.QRect(640, 40, 16, 16))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_15.setFont(font)
        self.label_15.setObjectName("label_15")
        self.modelPolarError = QtWidgets.QLineEdit(self.frame)
        self.modelPolarError.setEnabled(False)
        self.modelPolarError.setGeometry(QtCore.QRect(270, 35, 66, 25))
        self.modelPolarError.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.modelPolarError.setFont(font)
        self.modelPolarError.setMouseTracking(False)
        self.modelPolarError.setAcceptDrops(False)
        self.modelPolarError.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.modelPolarError.setText("")
        self.modelPolarError.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.modelPolarError.setReadOnly(True)
        self.modelPolarError.setObjectName("modelPolarError")
        self.label_11 = QtWidgets.QLabel(self.frame)
        self.label_11.setGeometry(QtCore.QRect(5, 35, 72, 25))
        self.label_11.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_11.setFont(font)
        self.label_11.setObjectName("label_11")
        self.solver = QtWidgets.QLineEdit(self.frame)
        self.solver.setEnabled(False)
        self.solver.setGeometry(QtCore.QRect(760, 35, 146, 25))
        self.solver.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.solver.setFont(font)
        self.solver.setMouseTracking(False)
        self.solver.setAcceptDrops(False)
        self.solver.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.solver.setText("")
        self.solver.setReadOnly(True)
        self.solver.setObjectName("solver")
        self.modelErrorRMS = QtWidgets.QLineEdit(self.frame)
        self.modelErrorRMS.setEnabled(False)
        self.modelErrorRMS.setGeometry(QtCore.QRect(90, 35, 46, 25))
        self.modelErrorRMS.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.modelErrorRMS.setFont(font)
        self.modelErrorRMS.setMouseTracking(False)
        self.modelErrorRMS.setAcceptDrops(False)
        self.modelErrorRMS.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.modelErrorRMS.setText("")
        self.modelErrorRMS.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.modelErrorRMS.setReadOnly(True)
        self.modelErrorRMS.setObjectName("modelErrorRMS")
        self.profile = QtWidgets.QLineEdit(self.frame)
        self.profile.setEnabled(False)
        self.profile.setGeometry(QtCore.QRect(465, 5, 171, 25))
        self.profile.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.profile.setFont(font)
        self.profile.setMouseTracking(False)
        self.profile.setAcceptDrops(False)
        self.profile.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.profile.setText("")
        self.profile.setReadOnly(True)
        self.profile.setObjectName("profile")
        self.label_17 = QtWidgets.QLabel(self.frame)
        self.label_17.setGeometry(QtCore.QRect(910, 5, 54, 25))
        self.label_17.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_17.setFont(font)
        self.label_17.setObjectName("label_17")
        self.goodPoints = QtWidgets.QLineEdit(self.frame)
        self.goodPoints.setEnabled(False)
        self.goodPoints.setGeometry(QtCore.QRect(965, 5, 26, 25))
        self.goodPoints.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.goodPoints.setFont(font)
        self.goodPoints.setMouseTracking(False)
        self.goodPoints.setAcceptDrops(False)
        self.goodPoints.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.goodPoints.setText("")
        self.goodPoints.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.goodPoints.setReadOnly(True)
        self.goodPoints.setObjectName("goodPoints")
        self.totalPoints = QtWidgets.QLineEdit(self.frame)
        self.totalPoints.setEnabled(False)
        self.totalPoints.setGeometry(QtCore.QRect(1000, 5, 26, 25))
        self.totalPoints.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Courier")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.totalPoints.setFont(font)
        self.totalPoints.setMouseTracking(False)
        self.totalPoints.setAcceptDrops(False)
        self.totalPoints.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.totalPoints.setText("")
        self.totalPoints.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.totalPoints.setReadOnly(True)
        self.totalPoints.setObjectName("totalPoints")
        self.label_22 = QtWidgets.QLabel(self.frame)
        self.label_22.setGeometry(QtCore.QRect(995, 5, 16, 25))
        self.label_22.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_22.setFont(font)
        self.label_22.setObjectName("label_22")
        self.verticalLayout.addWidget(self.frame)
        self.line = QtWidgets.QFrame(AnalyseDialog)
        self.line.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line.setLineWidth(2)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.groupBox_7 = QtWidgets.QGroupBox(AnalyseDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_7.sizePolicy().hasHeightForWidth())
        self.groupBox_7.setSizePolicy(sizePolicy)
        self.groupBox_7.setProperty("large", True)
        self.groupBox_7.setObjectName("groupBox_7")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.groupBox_7)
        self.gridLayout_8.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_8.setSpacing(0)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.modelPositions = QtWidgets.QWidget(self.groupBox_7)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.modelPositions.sizePolicy().hasHeightForWidth())
        self.modelPositions.setSizePolicy(sizePolicy)
        self.modelPositions.setToolTip("")
        self.modelPositions.setObjectName("modelPositions")
        self.gridLayout_8.addWidget(self.modelPositions, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_7, 0, 0, 1, 1)
        self.groupBox_2 = QtWidgets.QGroupBox(AnalyseDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_2.sizePolicy().hasHeightForWidth())
        self.groupBox_2.setSizePolicy(sizePolicy)
        self.groupBox_2.setProperty("large", True)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_3.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_3.setSpacing(0)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.scaleImage = QtWidgets.QWidget(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scaleImage.sizePolicy().hasHeightForWidth())
        self.scaleImage.setSizePolicy(sizePolicy)
        self.scaleImage.setToolTip("")
        self.scaleImage.setObjectName("scaleImage")
        self.gridLayout_3.addWidget(self.scaleImage, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_2, 0, 3, 1, 1)
        self.groupBox_8 = QtWidgets.QGroupBox(AnalyseDialog)
        self.groupBox_8.setProperty("large", True)
        self.groupBox_8.setObjectName("groupBox_8")
        self.gridLayout_9 = QtWidgets.QGridLayout(self.groupBox_8)
        self.gridLayout_9.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_9.setSpacing(0)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.errorAscending = QtWidgets.QWidget(self.groupBox_8)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.errorAscending.sizePolicy().hasHeightForWidth())
        self.errorAscending.setSizePolicy(sizePolicy)
        self.errorAscending.setToolTip("")
        self.errorAscending.setObjectName("errorAscending")
        self.gridLayout_9.addWidget(self.errorAscending, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_8, 0, 2, 1, 1)
        self.groupBox_5 = QtWidgets.QGroupBox(AnalyseDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_5.sizePolicy().hasHeightForWidth())
        self.groupBox_5.setSizePolicy(sizePolicy)
        self.groupBox_5.setProperty("large", True)
        self.groupBox_5.setObjectName("groupBox_5")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBox_5)
        self.gridLayout_6.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_6.setSpacing(0)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.decPointErrorsRaw = QtWidgets.QWidget(self.groupBox_5)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.decPointErrorsRaw.sizePolicy().hasHeightForWidth())
        self.decPointErrorsRaw.setSizePolicy(sizePolicy)
        self.decPointErrorsRaw.setToolTip("")
        self.decPointErrorsRaw.setObjectName("decPointErrorsRaw")
        self.gridLayout_6.addWidget(self.decPointErrorsRaw, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_5, 2, 0, 1, 1)
        self.groupBox_3 = QtWidgets.QGroupBox(AnalyseDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_3.sizePolicy().hasHeightForWidth())
        self.groupBox_3.setSizePolicy(sizePolicy)
        self.groupBox_3.setProperty("large", True)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_5.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_5.setSpacing(0)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.raPointErrorsRaw = QtWidgets.QWidget(self.groupBox_3)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.raPointErrorsRaw.sizePolicy().hasHeightForWidth())
        self.raPointErrorsRaw.setSizePolicy(sizePolicy)
        self.raPointErrorsRaw.setToolTip("")
        self.raPointErrorsRaw.setObjectName("raPointErrorsRaw")
        self.gridLayout_5.addWidget(self.raPointErrorsRaw, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_3, 1, 0, 1, 1)
        self.groupBox_9 = QtWidgets.QGroupBox(AnalyseDialog)
        self.groupBox_9.setProperty("large", True)
        self.groupBox_9.setObjectName("groupBox_9")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.groupBox_9)
        self.gridLayout_10.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_10.setSpacing(0)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.errorDistribution = QtWidgets.QWidget(self.groupBox_9)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.errorDistribution.sizePolicy().hasHeightForWidth())
        self.errorDistribution.setSizePolicy(sizePolicy)
        self.errorDistribution.setToolTip("")
        self.errorDistribution.setObjectName("errorDistribution")
        self.gridLayout_10.addWidget(self.errorDistribution, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_9, 0, 1, 1, 1)
        self.groupBox_4 = QtWidgets.QGroupBox(AnalyseDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_4.sizePolicy().hasHeightForWidth())
        self.groupBox_4.setSizePolicy(sizePolicy)
        self.groupBox_4.setProperty("large", True)
        self.groupBox_4.setObjectName("groupBox_4")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBox_4)
        self.gridLayout_4.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_4.setSpacing(0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.raModelErrors = QtWidgets.QWidget(self.groupBox_4)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.raModelErrors.sizePolicy().hasHeightForWidth())
        self.raModelErrors.setSizePolicy(sizePolicy)
        self.raModelErrors.setToolTip("")
        self.raModelErrors.setObjectName("raModelErrors")
        self.gridLayout_4.addWidget(self.raModelErrors, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_4, 1, 1, 1, 1)
        self.groupBox_10 = QtWidgets.QGroupBox(AnalyseDialog)
        self.groupBox_10.setProperty("large", True)
        self.groupBox_10.setObjectName("groupBox_10")
        self.gridLayout_11 = QtWidgets.QGridLayout(self.groupBox_10)
        self.gridLayout_11.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_11.setSpacing(0)
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.raModelErrorsRef = QtWidgets.QWidget(self.groupBox_10)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.raModelErrorsRef.sizePolicy().hasHeightForWidth())
        self.raModelErrorsRef.setSizePolicy(sizePolicy)
        self.raModelErrorsRef.setToolTip("")
        self.raModelErrorsRef.setObjectName("raModelErrorsRef")
        self.gridLayout_11.addWidget(self.raModelErrorsRef, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_10, 1, 2, 1, 1)
        self.groupBox_12 = QtWidgets.QGroupBox(AnalyseDialog)
        self.groupBox_12.setProperty("large", True)
        self.groupBox_12.setObjectName("groupBox_12")
        self.gridLayout_13 = QtWidgets.QGridLayout(self.groupBox_12)
        self.gridLayout_13.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_13.setSpacing(0)
        self.gridLayout_13.setObjectName("gridLayout_13")
        self.raPointErrorsRawRef = QtWidgets.QWidget(self.groupBox_12)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.raPointErrorsRawRef.sizePolicy().hasHeightForWidth())
        self.raPointErrorsRawRef.setSizePolicy(sizePolicy)
        self.raPointErrorsRawRef.setToolTip("")
        self.raPointErrorsRawRef.setObjectName("raPointErrorsRawRef")
        self.gridLayout_13.addWidget(self.raPointErrorsRawRef, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_12, 1, 3, 1, 1)
        self.groupBox_6 = QtWidgets.QGroupBox(AnalyseDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox_6.sizePolicy().hasHeightForWidth())
        self.groupBox_6.setSizePolicy(sizePolicy)
        self.groupBox_6.setProperty("large", True)
        self.groupBox_6.setObjectName("groupBox_6")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.groupBox_6)
        self.gridLayout_7.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_7.setSpacing(0)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.decModelErrors = QtWidgets.QWidget(self.groupBox_6)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.decModelErrors.sizePolicy().hasHeightForWidth())
        self.decModelErrors.setSizePolicy(sizePolicy)
        self.decModelErrors.setToolTip("")
        self.decModelErrors.setObjectName("decModelErrors")
        self.gridLayout_7.addWidget(self.decModelErrors, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_6, 2, 1, 1, 1)
        self.groupBox_13 = QtWidgets.QGroupBox(AnalyseDialog)
        self.groupBox_13.setProperty("large", True)
        self.groupBox_13.setObjectName("groupBox_13")
        self.gridLayout_14 = QtWidgets.QGridLayout(self.groupBox_13)
        self.gridLayout_14.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_14.setSpacing(0)
        self.gridLayout_14.setObjectName("gridLayout_14")
        self.decPointErrorsRawRef = QtWidgets.QWidget(self.groupBox_13)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.decPointErrorsRawRef.sizePolicy().hasHeightForWidth())
        self.decPointErrorsRawRef.setSizePolicy(sizePolicy)
        self.decPointErrorsRawRef.setToolTip("")
        self.decPointErrorsRawRef.setObjectName("decPointErrorsRawRef")
        self.gridLayout_14.addWidget(self.decPointErrorsRawRef, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_13, 2, 2, 1, 1)
        self.groupBox_11 = QtWidgets.QGroupBox(AnalyseDialog)
        self.groupBox_11.setProperty("large", True)
        self.groupBox_11.setObjectName("groupBox_11")
        self.gridLayout_12 = QtWidgets.QGridLayout(self.groupBox_11)
        self.gridLayout_12.setContentsMargins(4, 6, 0, 0)
        self.gridLayout_12.setSpacing(0)
        self.gridLayout_12.setObjectName("gridLayout_12")
        self.decModelErrorsRef = QtWidgets.QWidget(self.groupBox_11)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.decModelErrorsRef.sizePolicy().hasHeightForWidth())
        self.decModelErrorsRef.setSizePolicy(sizePolicy)
        self.decModelErrorsRef.setToolTip("")
        self.decModelErrorsRef.setObjectName("decModelErrorsRef")
        self.gridLayout_12.addWidget(self.decModelErrorsRef, 0, 0, 1, 1)
        self.gridLayout.addWidget(self.groupBox_11, 2, 3, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        self.verticalLayout.setStretch(2, 1)

        self.retranslateUi(AnalyseDialog)
        QtCore.QMetaObject.connectSlotsByName(AnalyseDialog)

    def retranslateUi(self, AnalyseDialog):
        _translate = QtCore.QCoreApplication.translate
        AnalyseDialog.setWindowTitle(_translate("AnalyseDialog", "Model Build File Analysis"))
        self.version.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.filename.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.subframe.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_20.setText(_translate("AnalyseDialog", "mm"))
        self.label_21.setText(_translate("AnalyseDialog", "/"))
        self.label_13.setText(_translate("AnalyseDialog", "arcsec"))
        self.modelOrthoError.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_18.setText(_translate("AnalyseDialog", "Profile"))
        self.pointsWest.setText(_translate("AnalyseDialog", "Pierside West"))
        self.pointsWest.setProperty("color", _translate("AnalyseDialog", "green"))
        self.pointsEast.setText(_translate("AnalyseDialog", "Pierside East"))
        self.pointsEast.setProperty("color", _translate("AnalyseDialog", "yellow"))
        self.label_5.setText(_translate("AnalyseDialog", "Subframe"))
        self.label_3.setText(_translate("AnalyseDialog", "Binning"))
        self.firmware.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_10.setText(_translate("AnalyseDialog", "Ortho Error"))
        self.time.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_4.setText(_translate("AnalyseDialog", "Time [UTC]"))
        self.label_9.setText(_translate("AnalyseDialog", "Polar Error"))
        self.label_7.setText(_translate("AnalyseDialog", "Mirrored"))
        self.label_6.setText(_translate("AnalyseDialog", "Build / FW"))
        self.load.setToolTip(_translate("AnalyseDialog", "Loads the model points from file"))
        self.load.setText(_translate("AnalyseDialog", "Load"))
        self.eposure.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_19.setText(_translate("AnalyseDialog", "Focal Length"))
        self.label_2.setText(_translate("AnalyseDialog", "Solver"))
        self.label_16.setText(_translate("AnalyseDialog", "s"))
        self.winsorizedLimit.setText(_translate("AnalyseDialog", "Winzorized"))
        self.modelTerms.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.binning.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_12.setText(_translate("AnalyseDialog", "arcsec"))
        self.focalLength.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_8.setText(_translate("AnalyseDialog", "Terms"))
        self.mirrored.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label.setText(_translate("AnalyseDialog", "Exposure"))
        self.label_14.setText(_translate("AnalyseDialog", "arcsec"))
        self.label_15.setText(_translate("AnalyseDialog", "%"))
        self.modelPolarError.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_11.setText(_translate("AnalyseDialog", "Model RMS"))
        self.solver.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.modelErrorRMS.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.profile.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_17.setText(_translate("AnalyseDialog", "Points"))
        self.goodPoints.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.totalPoints.setToolTip(_translate("AnalyseDialog", "Name of Model Points file, where the data is stored. "))
        self.label_22.setText(_translate("AnalyseDialog", "/"))
        self.groupBox_7.setTitle(_translate("AnalyseDialog", "Model errors overview in hemisphere"))
        self.groupBox_2.setTitle(_translate("AnalyseDialog", "Image pixel resolution as scale "))
        self.groupBox_8.setTitle(_translate("AnalyseDialog", "Model point errors in ascending order"))
        self.groupBox_5.setTitle(_translate("AnalyseDialog", "Declination - Raw Error (PlateSolve)"))
        self.groupBox_3.setTitle(_translate("AnalyseDialog", "Right Ascension - Raw Error (PlateSolve)"))
        self.groupBox_9.setTitle(_translate("AnalyseDialog", "Model point errors over polar"))
        self.groupBox_4.setTitle(_translate("AnalyseDialog", "Right Ascension - Model Error (Mount)"))
        self.groupBox_10.setTitle(_translate("AnalyseDialog", "Right Ascension - Model Error over Encoder value"))
        self.groupBox_12.setTitle(_translate("AnalyseDialog", "Right Ascension - Raw Error over Encoder value"))
        self.groupBox_6.setTitle(_translate("AnalyseDialog", "Declination - Model Error (Mount)"))
        self.groupBox_13.setTitle(_translate("AnalyseDialog", "Declination - Raw Error over Encoder value"))
        self.groupBox_11.setTitle(_translate("AnalyseDialog", "Declination - Model Error over Encoder value"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    AnalyseDialog = QtWidgets.QWidget()
    ui = Ui_AnalyseDialog()
    ui.setupUi(AnalyseDialog)
    AnalyseDialog.show()
    sys.exit(app.exec_())



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
# Licence APL2.0
#
###########################################################
# standard libraries
import socket
import logging
import uuid

# external packages

# local imports


class Connection(object):
    """
    The class Connection provides the command and reply interface to a 10 micron mount.
    There should be all commands and their return values be sent to the mount via
    IP and the responses.

    Define the number of chunks for the return bytes in case of not having them in
    bulk mode this is needed, because the mount computer  doesn't support a
    transaction base like number of chunks to be expected. It's just plain data and
    I have to find out myself how much it is. there are three types of commands:

          a) no reply               this is ok -> COMMAND_A
          b) reply without '#'      this is the bad part, don't like it -> COMMAND_B
          c) reply ended with '#'   this is normal feedback -> no special treatment

    The class itself need parameters for the host and port to be able to interact
    with the mount.

        >>> command = Connection(
        >>>                   host=('127.0.0.1', 3492),
        >>>                   )

    """

    __all__ = ['Connection',
               ]

    log = logging.getLogger(__name__)

    # I don't want so wait to long for a response. In average I see values
    # shorter than 0.5 sec, so 2 seconds should be good
    SOCKET_TIMEOUT = 3

    # complete used command list to be checked first if valid
    # these are the commands, which were used in mountcontrol so far
    COMMANDS = [':AP',
                ':FLIP',
                ':GDUT', ':GDUTV', ':GMs', ':GMsa', ':GMsb', ':GMACW', ':GMAC',
                ':GREF', ':GRPRS', ':GRTMP',
                ':GS', ':GT', ':GTMP1', ':GVD', ':GVN', ':GVP', ':GVT', ':GVZ',
                ':GWOL', ':Ga', ':GaXa', ':GaXb',
                ':Gd', ':Gdat', ':Gev', ':Gg', ':Gh', ':GINQ', ':Ginfo', ':Glms',
                ':Glmt', ':Gmte', ':Go', ':Gr', ':Gt', ':gtg', ':GTsid',
                ':Guaf', ':Gz',
                ':MA', ':MS', ':MaX', ':Me', ':Mn', ':Ms', ':Mw',
                ':MSao', ':MSap',
                ':PO', ':PO', ':PO', ':PaX', ':PiP',
                ':Q', ':QaXa', ':QaXb', ':Qe', ':Qn', ':Qs', ':Qw',
                ':RC', ':Rc', ':RG', ':RM', ':RMs', ':RS',
                ':RT0', ':RT1', ':RT2', ':RT9',
                ':SREF', ':SRPRS', ':SRTMP', ':STOP', ':SWOL',
                ':Sa', ':SaXa', ':SaXb', ':Sd',
                ':Sdat', ':Sev', ':Sev', ':Sg', ':Sg', ':Sh', ':Slms', ':Slmt',
                ':So', ':Sr', ':St', ':Suaf', ':Sw', ':Sz',
                ':TLEG', ':TLEL0', ':TLEGAZ', ':TLEGEQ', ':TLEP', ':TLES',
                ':TLESCK',
                ':U2',
                ':delalig', ':delalst',
                ':endalig',
                ':getain', ':getalp', ':getalst',
                ':hP',
                ':modelcnt', ':modeldel0', ':modelld0', ':modelnam',
                ':modelsv0',
                ':newalig', ':newalpt',
                ':WSG', ':WSP', ':WST', ':WSH', ':WSD', ':WSS',
                ':shutdown',
                ]

    # Command list for commands which don't reply anything
    COMMAND_A = [':AP', ':hP',
                 ':Me', ':Mn', ':Ms', ':Mw',
                 ':PO',
                 ':Q', ':Qe', ':Qn', ':Qs', ':Qw',
                 ':RC', ':Rc', ':RG', ':RM', ':RS', ':RT0', ':RT1', ':RT2',
                 ':RT9',
                 ':STOP',
                 ':U2',
                 ':hP', ':Suaf',
                 ]

    # Command list for commands which have a response, but have no end mark
    # mostly these commands response value of '0' or '1'
    COMMAND_B = [':FLIP', ':shutdown', ':GREF', ':Guaf',
                 ':MA',
                 ':RMs',
                 ':SREF', ':SRPRS', ':Sa', ':Sev', ':Sr',
                 ':SRTMP', ':Slmt', ':Slms', ':St', ':Sg', ':Sw', ':Sz',
                 ':Sdat', ':Gdat',
                 ':GTsid', ':So', ':Sh', ':Sd', ':MSap', ':MSao', ':MS',
                 ':WSS', ':SWOL'
                 ]

    def __init__(self,
                 host=None,
                 ):

        self.host = host
        self.id = str(uuid.uuid4())[:8]

    def validCommand(self, command):
        """
        validCommand test if command is valid and known.

        :param command: command for 10 micron to test
        :return: True if valid commands were issued
        """

        for key in sorted(self.COMMANDS, reverse=True):
            if command.startswith(key):
                return True
        return False

    def validCommandSet(self, commandString):
        """
        validCommandSet test if all commands in the commandString are valid and known.

        :param commandString: command for 10 micron to test
        :return: True if valid commands were issued
        """

        commandSet = commandString.split('#')[:-1]
        for command in commandSet:
            if not self.validCommand(command):
                return False
        return True

    def analyseCommand(self, commandString):
        """
        analyseCommand parses the provided commandString against the two command
        type A and B to evaluate if a response is expected and how many chunks of
        data show be received.

        the command slots will be sorted in reverse order to ensure that longer
        commands with the same leading characters will be tested first. otherwise
        the test will be ended before testing al commands.

        :param commandString:       string sent to the mount
        :return: chunksToReceive:   counted chunks
                 noResponse:        True, if we should not wait for receiving data
        """

        chunksToReceive = 0
        getData = False
        commandSet = commandString.split('#')[:-1]
        minBytes = 0
        for command in commandSet:
            foundCOMMAND_A = False
            for key in sorted(self.COMMAND_A, reverse=True):
                if command.startswith(key):
                    if key == ':Q':
                        continue
                    foundCOMMAND_A = True
                    break
            if not foundCOMMAND_A:
                getData = True
                for keyBad in sorted(self.COMMAND_B, reverse=True):
                    if command.startswith(keyBad):
                        minBytes += 1
                        break
                else:
                    chunksToReceive += 1
        return chunksToReceive, getData, minBytes

    def closeClientHard(self, client):
        """
        closeClientHard tries to shutdown a socket in case of error hard

        :param client:
        :return: success
        """
        if not client:
            return False

        try:
            # client.shutdown(socket.SHUT_RDWR)
            client.close()

        except Exception as e:
            self.log.warning(f'hard close: {e}')
            return False

        return True

    def buildClient(self):
        """
        buildClient checks necessary information and tries to open a socket. if success
        it returns the client (socket connection)

        :return: client for socket connection if succeeded
        """
        if not self.host:
            self.log.info(f'[{self.id}] no host defined')
            return None

        if not isinstance(self.host, tuple):
            self.log.info(f'[{self.id}] host entry malformed')
            return None

        client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        client.settimeout(self.SOCKET_TIMEOUT)
        client.setsockopt(socket.IPPROTO_TCP, socket.TCP_NODELAY, 1)

        try:
            client.connect(self.host)

        except socket.timeout:
            self.closeClientHard(client)
            self.log.debug(f'[{self.id}] socket timeout')
            return None

        except socket.error as e:
            self.closeClientHard(client)
            self.log.debug(f'[{self.id}] socket error: {e}')
            return None

        except Exception as e:
            self.closeClientHard(client)
            self.log.debug(f'[{self.id}] socket error: {e}')
            return None

        else:
            return client

    def sendData(self, client=None, commandString=''):
        """
        sendData sends all data of the command string out to the given socket client.

        :param client: socket client
        :param commandString: command to mount as string
        :return: success
        """
        try:
            client.sendall(commandString.encode())

        except Exception as e:
            self.closeClientHard(client)
            self.log.debug(f'[{self.id}] socket error: {e}')
            return False

        else:
            return True

    def receiveData(self, client=None, numberOfChunks=0, minBytes=0):
        """
        receive Data waits on the give socket client for a number of chunks to be receive or
        a minimum set of bytes received. the chunks are delimited with #. the min bytes is
        necessary, because the mount computer has commands which giv a response without
        delimiter. this is bad, but status.

        :param client: socket client
        :param numberOfChunks: number of data chunks
        :param minBytes: minimum number of data bytes
        :return: success and response data
        """

        response = ''
        try:
            while True:
                chunkRaw = client.recv(2048)
                try:
                    chunk = chunkRaw.decode('ASCII')

                except Exception as e:
                    self.log.warning(f'[{self.id}] {e}, {chunkRaw}')
                    return False, ''

                if not chunk:
                    break

                response += chunk
                if numberOfChunks == 0 and len(response) == minBytes:
                    break

                elif numberOfChunks != 0 and numberOfChunks == response.count('#'):
                    break

        except socket.timeout:
            self.log.debug(f'[{self.id}] socket timeout')
            return False, response

        except Exception as e:
            self.log.debug(f'[{self.id}] socket error: {e}')
            return False, response

        else:
            response = response.rstrip('#').split('#')
            self.log.trace(f'[{self.id}] response : {response}')
            return True, response

    def communicate(self, commandString):
        """
        transfer open a socket to the mount, takes the command string for the mount,
        analyses it, check validity and finally if valid sends it to the mount. If
        response expected, wait for the response and returns the data.

        :param commandString:
        :return: success:           True or False for full transfer
                 response:          the data load
                 numberOfChunks:    number of responses chunks which were split with #
        """

        if not self.validCommandSet(commandString):
            self.log.warning(f'[{self.id}] unknown commands: {commandString}')
            return False, 'wrong commands', 0

        numberOfChunks, getData, minBytes = self.analyseCommand(commandString)
        logFormat = '[{0}] sending  : {1}, getData: {2}, minBytes: {3}, chunks: {4}, host: {5}'
        self.log.trace(logFormat.format(self.id,
                                        commandString,
                                        getData,
                                        minBytes,
                                        numberOfChunks,
                                        self.host,
                                        ))

        client = self.buildClient()
        if client is None:
            return False, '', numberOfChunks

        suc = self.sendData(client=client,
                            commandString=commandString)
        if not suc:
            return False, '', numberOfChunks

        if not getData:
            self.closeClientHard(client)
            return True, '', numberOfChunks

        suc, response = self.receiveData(client=client,
                                         numberOfChunks=numberOfChunks,
                                         minBytes=minBytes)
        self.closeClientHard(client)
        del client

        return suc, response, numberOfChunks



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './mw4/gui/widgets/keypad.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_KeypadDialog(object):
    def setupUi(self, KeypadDialog):
        KeypadDialog.setObjectName("KeypadDialog")
        KeypadDialog.resize(240, 460)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(KeypadDialog.sizePolicy().hasHeightForWidth())
        KeypadDialog.setSizePolicy(sizePolicy)
        KeypadDialog.setMinimumSize(QtCore.QSize(240, 460))
        KeypadDialog.setMaximumSize(QtCore.QSize(310, 600))
        KeypadDialog.setSizeIncrement(QtCore.QSize(10, 10))
        KeypadDialog.setBaseSize(QtCore.QSize(10, 10))
        font = QtGui.QFont()
        font.setFamily("Arial")
        KeypadDialog.setFont(font)
        self.keypad = QtWidgets.QGridLayout(KeypadDialog)
        self.keypad.setContentsMargins(10, 10, 10, 10)
        self.keypad.setObjectName("keypad")

        self.retranslateUi(KeypadDialog)
        QtCore.QMetaObject.connectSlotsByName(KeypadDialog)

    def retranslateUi(self, KeypadDialog):
        _translate = QtCore.QCoreApplication.translate
        KeypadDialog.setWindowTitle(_translate("KeypadDialog", "Virtual Keypad"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    KeypadDialog = QtWidgets.QWidget()
    ui = Ui_KeypadDialog()
    ui.setupUi(KeypadDialog)
    KeypadDialog.show()
    sys.exit(app.exec_())



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
# Licence APL2.0
#
###########################################################
# standard libraries

# external packages

# local imports
from base.ascomClass import AscomClass


class CoverAscom(AscomClass):
    """
    the class Dome inherits all information and handling of the Dome device. there will be
    some parameters who will define the slewing position of the dome relating to the
    mount.dome = DomeAscom(app=None)
    """

    __all__ = ['CoverAscom',
               ]

    CYCLE_DEVICE = 3000
    CYCLE_DATA = 1000

    def __init__(self, app=None, signals=None, data=None):
        super().__init__(app=app, data=data, threadPool=app.threadPool)

        self.signals = signals
        self.data = data

    def getInitialConfig(self):
        """

        :return: true for test purpose
        """

        super().getInitialConfig()

        return True

    def workerPollData(self):
        """
        :return: true for test purpose
        """
        coverStates = ['NotPresent', 'Closed', 'Moving', 'Open', 'Unknown', 'Error']

        if not self.deviceConnected:
            return False

        state = self.client.CoverState
        stateText = coverStates[state]
        self.dataEntry(stateText, 'Status.Cover')

        return True

    def closeCover(self):
        """
        :return: true for test purpose
        """
        if not self.deviceConnected:
            return False

        self.client.CloseCover()
        return True

    def openCover(self):
        """
        :return: true for test purpose
        """
        if not self.deviceConnected:
            return False

        self.client.OpenCover()
        return True

    def haltCover(self):
        """
        :return: true for test purpose
        """
        if not self.deviceConnected:
            return False

        self.client.HaltCover()
        return True



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
# Licence APL2.0
#
###########################################################
# standard libraries
import logging

# external packages
from PyQt5.QtTest import QTest
from PyQt5.QtCore import QTimer

# local imports
from indibase import qtIndiBase


class IndiClass:
    """
    the class indiClass inherits all information and handling of indi devices
    this class will be only referenced from other classes and not directly used

        >>> indi = IndiClass(app=None, data=None)
    """

    __all__ = ['IndiClass']

    log = logging.getLogger(__name__)

    RETRY_DELAY = 1500
    NUMBER_RETRY = 5

    INDIGO = {
        # numbers
        'WEATHER_PARAMETERS.WEATHER_BAROMETER': 'WEATHER_PARAMETERS.WEATHER_PRESSURE',
        # SQM device
        'AUX_INFO.X_AUX_SKY_BRIGHTNESS': 'SKY_QUALITY.SKY_BRIGHTNESS',
        'AUX_INFO.X_AUX_SKY_TEMPERATURE': 'SKY_QUALITY.SKY_TEMPERATURE',
        # UPB device
        'AUX_INFO.X_AUX_AVERAGE': 'POWER_CONSUMPTION.CONSUMPTION_AVG_AMPS',
        'AUX_INFO.X_AUX_AMP_HOUR': 'POWER_CONSUMPTION.CONSUMPTION_AMP_HOURS',
        'AUX_INFO.X_AUX_WATT_HOUR': 'POWER_CONSUMPTION.CONSUMPTION_WATT_HOURS',
        'AUX_INFO.X_AUX_VOLTAGE': 'POWER_SENSORS.SENSOR_VOLTAGE',
        'AUX_INFO.X_AUX_CURRENT': 'POWER_SENSORS.SENSOR_CURRENT',
        'AUX_INFO.X_AUX_POWER_OUTLET': 'POWER_SENSORS.SENSOR_POWER',
        'AUX_POWER_OUTLET_CURRENT.OUTLET_1': 'POWER_CURRENT.POWER_CURRENT_1',
        'AUX_POWER_OUTLET_CURRENT.OUTLET_2': 'POWER_CURRENT.POWER_CURRENT_2',
        'AUX_POWER_OUTLET_CURRENT.OUTLET_3': 'POWER_CURRENT.POWER_CURRENT_3',
        'AUX_POWER_OUTLET_CURRENT.OUTLET_4': 'POWER_CURRENT.POWER_CURRENT_4',
        'AUX_HEATER_OUTLET_CURRENT.OUTLET_1': 'DEW_CURRENT.DEW_CURRENT_A',
        'AUX_HEATER_OUTLET_CURRENT.OUTLET_2': 'DEW_CURRENT.DEW_CURRENT_B',
        'AUX_HEATER_OUTLET_CURRENT.OUTLET_3': 'DEW_CURRENT.DEW_CURRENT_C',
        'AUX_HEATER_OUTLET.OUTLET_1': 'DEW_PWM.DEW_A',
        'AUX_HEATER_OUTLET.OUTLET_2': 'DEW_PWM.DEW_B',
        'AUX_HEATER_OUTLET.OUTLET_3': 'DEW_PWM.DEW_C',
        'X_AUX_VARIABLE_POWER_OUTLET.OUTLET_1': 'ADJUSTABLE_VOLTAGE.ADJUSTABLE_VOLTAGE_VALUE',
        # switches
        # UPB device
        'AUX_POWER_OUTLET.OUTLET_1': 'POWER_CONTROL.POWER_CONTROL_1',
        'AUX_POWER_OUTLET.OUTLET_2': 'POWER_CONTROL.POWER_CONTROL_2',
        'AUX_POWER_OUTLET.OUTLET_3': 'POWER_CONTROL.POWER_CONTROL_3',
        'AUX_POWER_OUTLET.OUTLET_4': 'POWER_CONTROL.POWER_CONTROL_4',
        'AUX_USB_PORT.PORT_1': 'USB_PORT_CONTROL.PORT_1',
        'AUX_USB_PORT.PORT_2': 'USB_PORT_CONTROL.PORT_2',
        'AUX_USB_PORT.PORT_3': 'USB_PORT_CONTROL.PORT_3',
        'AUX_USB_PORT.PORT_4': 'USB_PORT_CONTROL.PORT_4',
        'AUX_USB_PORT.PORT_5': 'USB_PORT_CONTROL.PORT_5',
        'AUX_USB_PORT.PORT_6': 'USB_PORT_CONTROL.PORT_6',
        'AUX_DEW_CONTROL.MANUAL': 'AUTO_DEW.INDI_DISABLED',
        'AUX_DEW_CONTROL.AUTOMATIC': 'AUTO_DEW.INDI_ENABLED',
        'X_AUX_REBOOT.REBOOT': 'REBOOT_DEVICE.REBOOT',
        # text
        # UPB device
        'X_AUX_OUTLET_NAMES.POWER_OUTLET_NAME_1': 'POWER_CONTROL_LABEL.POWER_LABEL_1',
        'X_AUX_OUTLET_NAMES.POWER_OUTLET_NAME_2': 'POWER_CONTROL_LABEL.POWER_LABEL_2',
        'X_AUX_OUTLET_NAMES.POWER_OUTLET_NAME_3': 'POWER_CONTROL_LABEL.POWER_LABEL_3',
        'X_AUX_OUTLET_NAMES.POWER_OUTLET_NAME_4': 'POWER_CONTROL_LABEL.POWER_LABEL_4',
    }

    INDI = {y: x for x, y in INDIGO.items()}

    INDI_TYPES = {
        'telescope': (1 << 0),
        'camera': (1 << 1),
        'guider': (1 << 2),
        'focuser': (1 << 3),
        'filterwheel': (1 << 4),
        'dome': (1 << 5),
        'observingconditions': (1 << 7) | (1 << 15),
        'skymeter': (1 << 15) | (1 << 19),
        'covercalibrator': (1 << 9) | (1 << 10),
        'power': (1 << 7) | (1 << 3) | (1 << 15) | (1 << 18),
    }

    def __init__(self, app=None, data=None, threadPool=None):
        super().__init__()

        self.app = app
        self.data = data
        self.client = qtIndiBase.Client(host=None, threadPool=threadPool)
        self.deviceName = ''
        self.device = None
        self._hostaddress = None
        self._host = None
        self._port = None
        self.retryCounter = 0
        self.discoverType = None
        self.discoverList = None
        self.loadIndiConfig = False
        self.isINDIGO = False
        self.showMessages = False

        self.defaultConfig = {
            'indi': {
                'deviceName': '',
                'deviceList': [],
                'hostaddress': 'localhost',
                'port': 7624,
                'loadConfig': False,
                'messages': False,
            }
        }

        self.timerRetry = QTimer()
        self.timerRetry.setSingleShot(True)
        self.timerRetry.timeout.connect(self.startRetry)

        self.client.signals.newDevice.connect(self.newDevice)
        self.client.signals.removeDevice.connect(self.removeDevice)
        self.client.signals.newProperty.connect(self.connectDevice)
        self.client.signals.newNumber.connect(self.updateNumber)
        self.client.signals.defNumber.connect(self.updateNumber)
        self.client.signals.newSwitch.connect(self.updateSwitch)
        self.client.signals.defSwitch.connect(self.updateSwitch)
        self.client.signals.newText.connect(self.updateText)
        self.client.signals.defText.connect(self.updateText)
        self.client.signals.newLight.connect(self.updateLight)
        self.client.signals.defLight.connect(self.updateLight)
        self.client.signals.newBLOB.connect(self.updateBLOB)
        self.client.signals.defBLOB.connect(self.updateBLOB)
        self.client.signals.deviceConnected.connect(self.loadConfig)
        self.client.signals.deviceConnected.connect(self.setUpdateConfig)
        self.client.signals.serverConnected.connect(self.serverConnected)
        self.client.signals.serverDisconnected.connect(self.serverDisconnected)
        self.client.signals.newMessage.connect(self.updateMessage)

    @property
    def host(self):
        return self._host

    @host.setter
    def host(self, value):
        self._host = value
        self.client.host = value

    @property
    def hostaddress(self):
        return self._hostaddress

    @hostaddress.setter
    def hostaddress(self, value):
        self._hostaddress = value
        self.client.host = (self._hostaddress, self._port)

    @property
    def port(self):
        return self._port

    @port.setter
    def port(self, value):
        self._port = int(value)
        self.client.host = (self._hostaddress, self._port)

    def serverConnected(self):
        """
        serverConnected is called when the server signals the connection. if
        so, we would like to start watching the defined device. this will be
        triggered directly

        :return: success
        """
        if self.deviceName:
            suc = self.client.watchDevice(self.deviceName)
            self.log.debug(f'INDI watch: {self.deviceName}, watch: result:{suc}')
            return suc
        return False

    def serverDisconnected(self, devices):
        """
        :param devices:
        :return: true for test purpose
        """
        self.log.debug(f'INDI server disconnected, devices: {devices.keys()}')
        return True

    def newDevice(self, deviceName):
        """
        newDevice is called whenever a new device entry is received in indi
        client. it adds the device if the name fits to the given name in
        configuration.

        :param deviceName:
        :return: true for test purpose
        """
        if deviceName == self.deviceName:
            self.device = self.client.getDevice(deviceName)
            self.app.message.emit(f'INDI device found:   [{deviceName}]', 0)
        else:
            self.log.info(f'INDI device snoop: {deviceName}')
        return True

    def removeDevice(self, deviceName):
        """
        removeDevice is called whenever a device is removed from indi client.
        it sets the device entry to None

        :param deviceName:
        :return: true for test purpose
        """
        if deviceName == self.deviceName:
            self.app.message.emit(f'INDI removed device: [{deviceName}]', 0)
            self.device = None
            self.data.clear()
            return True

        else:
            return False

    def startRetry(self):
        """
        startRetry tries to connect the server a NUMBER_RETRY times, if
        necessary with a delay of RETRY_DELAY

        :return: True for test purpose
        """
        if not self.deviceName:
            return False

        if self.data:
            return True

        self.retryCounter += 1
        suc = self.client.connectServer()
        if suc:
            return True

        t = f'Cannot start to: {self.deviceName} retry: {self.retryCounter}'
        self.log.debug(t)
        if self.retryCounter < self.NUMBER_RETRY:
            self.timerRetry.start(self.RETRY_DELAY)
        return True

    def startCommunication(self, loadConfig=False):
        """
        :param loadConfig:
        :return: success of reconnecting to server
        """
        self.loadIndiConfig = loadConfig
        self.retryCounter = 0
        self.client.startTimers()
        suc = self.client.connectServer()
        if not suc:
            t = f'Cannot start to: {self.deviceName} retry: {self.retryCounter}'
            self.log.debug(t)
        else:
            self.timerRetry.start(self.RETRY_DELAY)
        return suc

    def stopCommunication(self):
        """
        :return: success of reconnecting to server
        """
        self.client.stopTimers()
        suc = self.client.disconnectServer(self.deviceName)
        self.deviceName = ''
        return suc

    def connectDevice(self, deviceName, propertyName):
        """
        connectDevice is called when a new property is received and checks it
        against property CONNECTION. if this is there, we could check the
        connection state of a given device

        :param deviceName:
        :param propertyName:
        :return: success if device could connect
        """
        if propertyName != 'CONNECTION':
            return False

        suc = False
        if deviceName == self.deviceName:
            suc = self.client.connectDevice(deviceName=deviceName)
        return suc

    def loadConfig(self, deviceName):
        """
        loadConfig send the command to the indi server to load the default
        config for the given device.

        :param deviceName:
        :return: success
        """
        if not self.loadIndiConfig:
            return False

        loadObject = self.device.getSwitch('CONFIG_PROCESS')
        loadObject['CONFIG_LOAD'] = True
        suc = self.client.sendNewSwitch(deviceName=deviceName,
                                        propertyName='CONFIG_PROCESS',
                                        elements=loadObject,
                                        )
        return suc

    def setUpdateConfig(self, deviceName):
        """
        :param deviceName:
        :return: success
        """
        pass

    def convertIndigoProperty(self, key):
        """
        :param key:
        :return:
        """
        if key in self.INDIGO:
            key = self.INDIGO.get(key)
        return key

    def updateNumber(self, deviceName, propertyName):
        """
        updateNumber is called whenever a new number is received in client.
        it runs through the device list and writes the number data to the
        according locations.

        :param deviceName:
        :param propertyName:
        :return: success
        """
        if self.device is None:
            return False
        if deviceName != self.deviceName:
            return False

        for element, value in self.device.getNumber(propertyName).items():
            key = propertyName + '.' + element

            # print('number', self.deviceName, key, value)
            key = self.convertIndigoProperty(key)

            self.data[key] = float(value)

        return True

    def updateSwitch(self, deviceName, propertyName):
        """
        updateSwitch is called whenever a new switch is received in client.
        it runs through the device list and writes the switch data to the
        according locations.

        :param deviceName:
        :param propertyName:
        :return: success
        """
        if self.device is None:
            return False
        if deviceName != self.deviceName:
            return False

        for element, value in self.device.getSwitch(propertyName).items():
            key = propertyName + '.' + element

            # is that the item which tells me it's an indigo server ?
            if propertyName == 'PROFILE':
                self.isINDIGO = True

            # print('switch', self.deviceName, key, value)
            key = self.convertIndigoProperty(key)

            self.data[key] = value == 'On'

        return True

    def updateText(self, deviceName, propertyName):
        """
        updateText is called whenever a new text is received in client.
        it runs through the device list and writes the text data to the according
        locations.

        :param deviceName:
        :param propertyName:
        :return: success
        """
        if self.device is None:
            return False
        if deviceName != self.deviceName:
            return False

        for element, value in self.device.getText(propertyName).items():
            key = propertyName + '.' + element

            # print('text  ', self.deviceName, key, value)
            key = self.convertIndigoProperty(key)

            self.data[key] = value

        return True

    def updateLight(self, deviceName, propertyName):
        """
        updateLight is called whenever a new light is received in client.
        it runs through the device list and writes the light data to the
        according locations.

        :param deviceName:
        :param propertyName:
        :return: success
        """
        if self.device is None:
            return False
        if deviceName != self.deviceName:
            return False

        for element, value in self.device.getLight(propertyName).items():
            key = propertyName + '.' + element

            # print('light ', self.deviceName, key, value)
            key = self.convertIndigoProperty(key)

            self.data[key] = value

        return True

    def updateBLOB(self, deviceName, propertyName):
        """
        updateBLOB is called whenever a new BLOB is received in client.
        it runs through the device list and writes the BLOB data to the according
        locations.

        :param deviceName:
        :param propertyName:
        :return: success
        """
        if self.device is None:
            return False
        if deviceName != self.deviceName:
            return False
        return True

    @staticmethod
    def removePrefix(text, prefix):
        """
        :param text:
        :param prefix:
        :return:
        """
        value = text[text.startswith(prefix) and len(prefix):]
        value = value.strip()
        return value

    def updateMessage(self, device, text):
        """
        message take a message send by indi device and emits them in the user
        message window as well.

        :param device: device name
        :param text: message received
        :return: success
        """
        if self.showMessages:
            if text.startswith('[WARNING]'):
                text = self.removePrefix(text, '[WARNING]')
                self.app.message.emit(f'INDI {device:15s} {text}', 0)
            elif text.startswith('[ERROR]'):
                text = self.removePrefix(text, '[ERROR]')
                self.app.message.emit(f'INDI {device:15s} {text}', 2)
            else:
                self.app.message.emit(f'INDI {device:15s} {text}', 0)
            return True
        return False

    def addDiscoveredDevice(self, deviceName, propertyName):
        """
        addDevicesWithType gety called whenever a new device send out text
        messages. than it checks, if the device type fits to the search type
        desired. if they match, the device name is added to the list.

        unfortunately the indi definitions are not well defined. so for example
        SQM reports only aux general. this is value '0'. So i have to treat all
        devices reporting device type '0' as devices which could be used for
        everything.

        :param deviceName:
        :param propertyName:
        :return: success
        """
        if propertyName != 'DRIVER_INFO':
            return False

        device = self.client.devices.get(deviceName)
        if not device:
            return False

        interface = device.getText(propertyName).get('DRIVER_INTERFACE', None)
        if interface is None:
            return False

        if interface == '0':
            interface = 0xffff

        if self.discoverType is None:
            return False

        self.log.debug(f'Found: [{deviceName}], interface: [{interface}]')
        interface = int(interface)
        if interface & self.discoverType:
            self.discoverList.append(deviceName)
        return True

    def discoverDevices(self, deviceType=''):
        """
        discoverDevices implements a discover for devices of a certain device
        type. it is called from a button press and checks which button it was.
        after that for the right device it collects all necessary data for host
        value, instantiates an INDI client and watches for all devices connected
        to this server. Than it connects a subroutine for collecting the right
        device names and waits a certain amount of time. the data collection
        takes place as long as the model dialog is open. when the user closes
        this dialog, the collected data is written to the drop down list.

        :param deviceType: device type of discovered indi devices
        :return: success
        """
        self.discoverList = list()
        self.discoverType = self.INDI_TYPES.get(deviceType, 0)
        self.client.signals.defText.connect(self.addDiscoveredDevice)
        self.client.connectServer()
        self.client.watchDevice()
        QTest.qWait(2000)
        self.client.signals.defText.disconnect(self.addDiscoveredDevice)
        self.client.disconnectServer()
        return self.discoverList



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
#
# Licence APL2.0
#
###########################################################
# standard libraries
import logging
import datetime
import uuid
from dateutil.parser import parser

# external packages
from PyQt5.QtCore import QObject, pyqtSignal
import requests

# local imports


class AlpacaSignals(QObject):

    """
    The AlpacaSignals class offers a list of signals to be used and instantiated by
    the Alpaca class to get signals for triggers for finished tasks to
    enable a gui to update their values transferred to the caller back.

    This has to be done in a separate class as the signals have to be subclassed from
    QObject.
    """

    __all__ = ['AlpacaSignals']

    serverConnected = pyqtSignal()
    serverDisconnected = pyqtSignal(object)
    deviceConnected = pyqtSignal(str)
    deviceDisconnected = pyqtSignal(str)


class AlpacaBase:
    """
    the class AlpacaBase inherits all information and handling of alpaca devices
    this class will be only referenced from other classes and will be not used
    directly

        >>> a = AlpacaBase()
    """

    __all__ = ['AlpacaBase']

    log = logging.getLogger(__name__)

    def __init__(self):
        super().__init__()

        self.signals = AlpacaSignals()

        self.protocol = 'http'
        self.host = ('localhost', 11111)
        self.apiVersion = 1
        self.deviceName = ''
        self.deviceType = ''
        self.number = 0

    def generateBaseUrl(self):
        """
        :return: value for base url
        """

        val = '{0}://{1}:{2}/api/v{3}/{4}/{5}'.format(
            self.protocol,
            self.host[0],
            self.host[1],
            self.apiVersion,
            self.deviceType,
            self.number,
        )
        return val

    @property
    def baseUrl(self):
        return self.generateBaseUrl()

    @property
    def deviceName(self):
        return self._deviceName

    @deviceName.setter
    def deviceName(self, value):
        self._deviceName = value
        valueSplit = value.split(':')
        if len(valueSplit) != 3:
            return
        self.deviceType = valueSplit[1].strip()
        self.number = valueSplit[2].strip()
        self.number = int(self.number)

    def discoverAPIVersion(self):
        """

        :return:
        """

        self.log.trace('get api version')

        url = '{0}://{1}:{2}/management/apiversions'.format(
            self.protocol,
            self.host[0],
            self.host[1],
        )

        try:
            response = requests.get(url, timeout=3)

        except requests.exceptions.Timeout:
            self.log.info('timeout')
            return None

        except requests.exceptions.ConnectionError:
            self.log.debug('[connection error')
            return None

        except Exception as e:
            self.log.critical(f'[error in request: {e}')
            return None

        if response.status_code == 400 or response.status_code == 500:
            self.log.debug(f'{response.text}')
            return None

        response = response.json()

        if response['ErrorNumber'] != 0:
            self.log.warning(f'{response} err:{response["ErrorNumber"]}'
                             f',{response["ErrorMessage"]}')
            return None

        self.log.trace(f'[response:{response}')

        return response['Value']

    def discoverDevices(self):
        """

        :return:
        """

        self.log.trace('discover devices')

        url = '{0}://{1}:{2}/management/v{3}/configureddevices'.format(
            self.protocol,
            self.host[0],
            self.host[1],
            self.apiVersion,
        )

        try:
            response = requests.get(url, timeout=3)

        except requests.exceptions.Timeout:
            self.log.info('timeout')
            return None

        except requests.exceptions.ConnectionError:
            self.log.debug('[connection error')
            return None

        except Exception as e:
            self.log.critical(f'[error in request: {e}')
            return None

        if response.status_code == 400 or response.status_code == 500:
            self.log.debug(f'{response.text}')
            return None

        response = response.json()

        if response['ErrorNumber'] != 0:
            self.log.warning(f'{response} err:{response["ErrorNumber"]}'
                             f',{response["ErrorMessage"]}')
            return None

        self.log.trace(f'[response:{response}')

        return response['Value']

    def get(self, attr: str, **data):
        """
        Send an HTTP GET request to an Alpaca server and check response for errors.
        In addition a unique Transaction ID is generated, to verify which send /
        response pair belongs together.

        :param
            attr (str): attr to get from server.
            **data: Data to send with request.

        """

        if not self.deviceName:
            return None

        uid = uuid.uuid4().int % 2**32
        data['ClientTransactionID'] = uid

        self.log.trace(f'[{uid:10d}] {self.baseUrl}, attr:[{attr}]')

        try:
            response = requests.get(f'{self.baseUrl}/{attr}', data=data, timeout=5)

        except requests.exceptions.Timeout:
            self.log.info(f'[{uid:10d}] timeout')
            return None

        except requests.exceptions.ConnectionError:
            self.log.debug(f'[{uid:10d}] connection error')
            return None

        except Exception as e:
            self.log.critical(f'[{uid:10d}] error in request: {e}')
            return None

        if response.status_code == 400 or response.status_code == 500:
            self.log.debug(f'{response.text}')
            return None

        response = response.json()

        if response['ErrorNumber'] != 0:
            self.log.warning(f'{response} err:{response["ErrorNumber"]}'
                             f',{response["ErrorMessage"]}')
            return None

        self.log.trace(f'[{uid:10d}] response:{response}')
        return response['Value']

    def put(self, attr: str, **data):
        """
        Send an HTTP PUT request to an Alpaca server and check response for errors.
        In addition a unique Transaction ID is generated, to verify which send /
        response pair belongs together.

        :param
            attr (str): attr to put to server.
            **data: Data to send with request.

        """
        if not self.deviceName:
            return None

        uid = uuid.uuid4().int % 2**32
        data['ClientTransactionID'] = uid

        self.log.trace(f'[{uid:08d}] {self.baseUrl}, attr:[{attr}]')

        try:
            response = requests.put(f'{self.baseUrl}/{attr}', data=data, timeout=5)

        except requests.exceptions.Timeout:
            self.log.info(f'[{uid:10d}] timeout')
            return None

        except requests.exceptions.ConnectionError:
            self.log.debug(f'[{uid:10d}] connection error')
            return None

        except Exception as e:
            self.log.critical(f'[{uid:10d}] Error in request: {e}')
            return None

        if response.status_code == 400 or response.status_code == 500:
            self.log.debug(f'[{uid:10d}] {response.text}')
            return None

        response = response.json()

        if response['ErrorNumber'] != 0:
            self.log.warning(f'err:{response["ErrorNumber"]},{response["ErrorMessage"]}')
            return None

        self.log.trace(f'[{uid:10d}] response:{response}')

        return response

    def action(self, Action: str, *Parameters):
        """
        Access functionality beyond the built-in capabilities of the ASCOM device interfaces.

        :param
            Action (str): A well known name that represents the action to be carried out.
            *Parameters: List of required parameters or empty if none are required.
        """

        return self.put('action', Action=Action, Parameters=Parameters)

    def commandblind(self, Command, Raw):
        """
        Transmit an arbitrary string to the device and does not wait for a response.
        :param
            Command (str): The literal command string to be transmitted.
            Raw (bool): If true, command is transmitted 'as-is'.
                If false, then protocol framing characters may be added prior to
                transmission.
        """

        self.put('commandblind', Command=Command, Raw=Raw)

    def commandbool(self, Command, Raw):
        """
        Transmit an arbitrary string to the device and wait for a boolean response.

        :param
            Command (str): The literal command string to be transmitted.
            Raw (bool): If true, command is transmitted 'as-is'.
                If false, then protocol framing characters may be added prior to
                transmission.
        """

        return self.put('commandbool', Command=Command, Raw=Raw)

    def commandstring(self, Command, Raw):
        """
        Transmit an arbitrary string to the device and wait for a string response.
        :param
            Command (str): The literal command string to be transmitted.
            Raw (bool): If true, command is transmitted 'as-is'.
                If false, then protocol framing characters may be added prior to
                transmission.
        """

        return self.put('commandstring', Command=Command, Raw=Raw)

    def connected(self, Connected=None):
        """
        Retrieve or set the connected state of the device.
        :param
            Connected (bool): Set True to connect to device hardware.
                Set False to disconnect from device hardware.
                Set None to get connected state (default).

        """

        if Connected is None:
            return self.get('connected')

        self.put('connected', Connected=Connected)

    def description(self):
        """
        Get description of the device.
        """

        return self.get('name')

    def driverInfo(self):
        """
        Get information of the device.
        """
        val = self.get('driverinfo')

        if val is None:
            return ''

        return [i.strip() for i in val.split(',')]

    def driverVersion(self):
        """
        Get string containing only the major and minor version of the driver.
        """

        return self.get('driverversion')

    def interfaceVersion(self):
        """
        ASCOM Device interface version number that this device supports.
        """

        return self.get('interfaceversion')

    def nameDevice(self):
        """
        Get name of the device.
        """
        return self.get('name')

    def supportedActions(self):
        """
        Get list of action names supported by this driver.
        """

        return self.get('supportedactions')


class Switch(AlpacaBase):
    """
    Switch specific methods.
    """

    __all__ = ['Switch']

    def maxswitch(self):
        """
        Count of switch devices managed by this driver.
        :return:
            Number of switch devices managed by this driver. Devices are numbered from 0
            to MaxSwitch - 1.
        """

        return self.get("maxswitch")

    def canwrite(self, Id=0):
        """
        Indicate whether the specified switch device can be written to.
        Notes:
            Devices are numbered from 0 to MaxSwitch - 1.
        :param
            Id (int): The device number.
        :return:
            Whether the specified switch device can be written to, default true. This is
            false if the device cannot be written to, for example a limit switch or a
            sensor.
        """

        return self.get("canwrite", Id=Id)

    def getswitch(self, Id=0):
        """
        Return the state of switch device id as a boolean.
        Notes:
            Devices are numbered from 0 to MaxSwitch - 1.
        :param: Id (int: The device number.

        :return:
            State of switch device id as a boolean.
        """

        return self.get("getswitch", Id=Id)

    def getswitchdescription(self, Id=0):
        """
        Get the description of the specified switch device.
        Notes:
            Devices are numbered from 0 to MaxSwitch - 1.
        :param
            Id (int): The device number.

        :return:
            Description of the specified switch device.
        """

        return self.get("getswitchdescription", Id=Id)

    def getswitchname(self, Id=0):
        """
        Get the name of the specified switch device.
        Notes:
            Devices are numbered from 0 to MaxSwitch - 1.
        :param
            Id (int): The device number.

        :return:
            Name of the specified switch device.
        """

        return self.get("getswitchname", Id=Id)

    def getswitchvalue(self, Id=0):
        """
        Get the value of the specified switch device as a double.
        Notes:
            Devices are numbered from 0 to MaxSwitch - 1.
        :param
            Id (int): The device number.

        :return:
            Value of the specified switch device.
        """

        return self.get("getswitchvalue", Id=Id)

    def minswitchvalue(self, Id=0):
        """
        Get the minimum value of the specified switch device as a double.
        Notes:
            Devices are numbered from 0 to MaxSwitch - 1.
        :param
            Id (int): The device number.

        :return:
            Minimum value of the specified switch device as a double.
        """

        return self.get("minswitchvalue", Id=Id)

    def setswitch(self, Id: int, State: bool):
        """
        Set a switch controller device to the specified state, True or False.
        Notes:
            Devices are numbered from 0 to MaxSwitch - 1.
        :param
            Id (int): The device number.
            State (bool): The required control state (True or False).
        """

        self.put("setswitch", Id=Id, State=State)

    def setswitchname(self, Id: int, Name: str):
        """
        Set a switch device name to the specified value.
        Notes:
            Devices are numbered from 0 to MaxSwitch - 1.
        :param
            Id (int): The device number.
            Name (str): The name of the device.
        """

        self.put("setswitchname", Id=Id, Name=Name)

    def setswitchvalue(self, Id: int, Value):
        """
        Set a switch device value to the specified value.
        Notes:
            Devices are numbered from 0 to MaxSwitch - 1.
        :param
            Id (int): The device number.
            Value (float): Value to be set, between MinSwitchValue and MaxSwitchValue.
        """

        self.put("setswitchvalue", Id=Id, Value=Value)

    def switchstep(self, Id=0):
        """
        Return the step size that this device supports.
        Return the step size that this device supports (the difference between
        successive values of the device).
        Notes:
            Devices are numbered from 0 to MaxSwitch - 1.
        :param
            Id (int): The device number.

        :return:
            Maximum value of the specified switch device as a double.
        """

        return self.get("switchstep", Id=Id)


class SafetyMonitor(AlpacaBase):
    """
    Safety monitor specific methods.
    """

    __all__ = ['SafetyMonitor']

    def issafe(self):
        """
        Indicate whether the monitored state is safe for use.
        :return:
            True if the state is safe, False if it is unsafe.
        """

        return self.get("issafe")


class Dome(AlpacaBase):
    """
    Dome specific methods.
    """

    __all__ = ['Dome']

    def altitude(self):
        """
        Dome altitude.
        :return:
            Dome altitude (degrees, horizon zero and increasing positive to 90 zenith).
        """

        return self.get("altitude")

    def athome(self):
        """
        Indicate whether the dome is in the home position.
        Notes:
            This is normally used following a findhome() operation. The value is reset
            with any azimuth slew operation that moves the dome away from the home
            position. athome() may also become true during normal slew operations, if the
            dome passes through the home position and the dome controller hardware is
            capable of detecting that; or at the end of a slew operation if the dome
            comes to rest at the home position.
        :return:
            True if dome is in the home position.
        """

        return self.get("athome")

    def atpark(self):
        """
        Indicate whether the telescope is at the park position.
        Notes:
            Set only following a park() operation and reset with any slew operation.
        :return:
            True if the dome is in the programmed park position.
        """

        return self.get("atpark")

    def azimuth(self):
        """
        Dome azimuth.
        :return:
            Dome azimuth (degrees, North zero and increasing clockwise, i.e., 90 East,
            180 South, 270 West).
        """

        return self.get("azimuth")

    def canfindhome(self):
        """
        Indicate whether the dome can find the home position.
        :return:
            True if the dome can move to the home position.
        """

        return self.get("canfindhome")

    def canpark(self):
        """
        Indicate whether the dome can be parked.
        :return:
            True if the dome is capable of programmed parking (park() method).
        """

        return self.get("canpark")

    def cansetaltitude(self):
        """
        Indicate whether the dome altitude can be set.
        :return:
            True if driver is capable of setting the dome altitude.
        """

        return self.get("cansetaltitude")

    def cansetazimuth(self):
        """
        Indicate whether the dome azimuth can be set.
        :return:
            True if driver is capable of setting the dome azimuth.
        """

        return self.get("cansetazimuth")

    def cansetpark(self):
        """
        Indicate whether the dome park position can be set.
        :return:
            True if driver is capable of setting the dome park position.
        """

        return self.get("cansetpark")

    def cansetshutter(self):
        """
        Indicate whether the dome shutter can be opened.
        :return:
            True if driver is capable of automatically operating shutter.
        """

        return self.get("cansetshutter")

    def canslave(self):
        """
        Indicate whether the dome supports slaving to a telescope.
        :return:
            True if driver is capable of slaving to a telescope.
        """

        return self.get("canslave")

    def cansyncazimuth(self):
        """
        Indicate whether the dome azimuth position can be synched.
        Notes:
            True if driver is capable of synchronizing the dome azimuth position using
            the synctoazimuth(float) method.

        :return:
            True or False value.
        """

        return self.get("cansyncazimuth")

    def shutterstatus(self):
        """
        Status of the dome shutter or roll-off roof.
        Notes:
            0 = Open, 1 = Closed, 2 = Opening, 3 = Closing, 4 = Shutter status error.

        :return:
            Status of the dome shutter or roll-off roof.
        """

        return self.get("shutterstatus")

    def slaved(self, Slaved=None):
        """
        Set or indicate whether the dome is slaved to the telescope.

        :return:
            True or False value in not set.
        """

        if Slaved is None:
            return self.get("slaved")

        self.put("slaved", Slaved=Slaved)

    def slewing(self):
        """
        Indicate whether the any part of the dome is moving.
        Notes:
            True if any part of the dome is currently moving, False if all dome
            components are steady.

        Return:
            True or False value.
        """

        return self.get("slewing")

    def abortslew(self):
        """
        Immediately cancel current dome operation.
        Notes:
            Calling this method will immediately disable hardware slewing (Slaved will
            become False).
        """

        self.put("abortslew")

    def closeshutter(self):
        """
        Close the shutter or otherwise shield telescope from the sky.
        """

        self.put("closeshutter")

    def findhome(self):
        """
        Start operation to search for the dome home position.
        Notes:
            After home position is established initializes azimuth to the default value
            and sets the athome flag.
        """

        self.put("findhome")

    def openshutter(self):
        """
        Open shutter or otherwise expose telescope to the sky.
        """

        self.put("openshutter")

    def park(self):
        """
        Rotate dome in azimuth to park position.
        Notes:
            After assuming programmed park position, sets atpark flag.
        """

        self.put("park")

    def setpark(self):
        """
        Set current azimuth, altitude position of dome to be the park position.
        """

        self.put("setpark")

    def slewtoaltitude(self, Altitude):
        """
        Slew the dome to the given altitude position.
        """

        self.put("slewtoaltitude", Altitude=Altitude)

    def slewtoazimuth(self, Azimuth):
        """
        Slew the dome to the given azimuth position.

        :param
            Azimuth (float): Target dome azimuth (degrees, North zero and increasing
                clockwise. i.e., 90 East, 180 South, 270 West).
        """

        self.put("slewtoazimuth", Azimuth=Azimuth)

    def synctoazimuth(self, Azimuth):
        """
        Synchronize the current position of the dome to the given azimuth.
        :param
            Azimuth (float): Target dome azimuth (degrees, North zero and increasing
                clockwise. i.e., 90 East, 180 South, 270 West).
        """

        self.put("synctoazimuth", Azimuth=Azimuth)


class Camera(AlpacaBase):
    """
    Camera specific methods.
    """

    __all__ = ['Camera']

    def bayeroffsetx(self):
        """
        Return the X offset of the Bayer matrix, as defined in SensorType.
        """

        return self.get("bayeroffsetx")

    def bayeroffsety(self):
        """
        Return the Y offset of the Bayer matrix, as defined in SensorType.
        """

        return self.get("bayeroffsety")

    def binx(self, BinX=None):
        """
        Set or return the binning factor for the X axis.
        :param
            BinX (int): The X binning value.

        :return:
            Binning factor for the X axis.
        """

        if BinX is None:
            return self.get("binx")

        self.put("binx", BinX=BinX)

    def biny(self, BinY=None):
        """
        Set or return the binning factor for the Y axis.
        :param
            BinY (int): The Y binning value.

        :return:
            Binning factor for the Y axis.
        """

        if BinY is None:
            return self.get("biny")

        self.put("biny", BinY=BinY)

    def camerastate(self):
        """
        Return the camera operational state.
        Notes:
            0 = CameraIdle, 1 = CameraWaiting, 2 = CameraExposing,
            3 = CameraReading, 4 = CameraDownload, 5 = CameraError.

        :return:
            Current camera operational state as an integer.
        """

        return self.get("camerastate")

    def cameraxsize(self):
        """
        Return the width of the CCD camera chip.
        """

        return self.get("cameraxsize")

    def cameraysize(self):
        """
        Return the height of the CCD camera chip.
        """

        return self.get("cameraysize")

    def canabortexposure(self):
        """
        Indicate whether the camera can abort exposures.
        """

        return self.get("canabortexposure")

    def canasymmetricbin(self):
        """
        Indicate whether the camera supports asymmetric binning.
        """

        return self.get("canasymmetricbin")

    def canfastreadout(self):
        """
        Indicate whether the camera has a fast readout mode.
        """

        return self.get("canfastreadout")

    def cangetcoolerpower(self):
        """
        Indicate whether the camera's cooler power setting can be read.
        """

        return self.get("cangetcoolerpower")

    def canpulseguide(self):
        """
        Indicate whether this camera supports pulse guiding.
        """

        return self.get("canpulseguide")

    def cansetccdtemperature(self):
        """
        Indicate whether this camera supports setting the CCD temperature.
        """

        return self.get("cansetccdtemperature")

    def canstopexposure(self):
        """
        Indicate whether this camera can stop an exposure that is in progress.
        """

        return self.get("canstopexposure")

    def ccdtemperature(self):
        """
        Return the current CCD temperature in degrees Celsius.
        """

        return self.get("ccdtemperature")

    def cooleron(self, CoolerOn=None):
        """
        Turn the camera cooler on and off or return the current cooler on/off state.
        Notes:
            True = cooler on, False = cooler off.
        :param
            CoolerOn (bool): Cooler state.
        :return:
            Current cooler on/off state.
        """

        if CoolerOn is None:
            return self.get("cooleron")

        self.put("cooleron", CoolerOn=CoolerOn)

    def coolerpower(self):
        """
        Return the present cooler power level, in percent.
        """

        return self.get("coolerpower")

    def electronsperadu(self):
        """
        Return the gain of the camera in photoelectrons per A/D unit.
        """

        return self.get("electronsperadu")

    def exposuremax(self):
        """
        Return the maximum exposure time supported by StartExposure.
        """

        return self.get("exposuremax")

    def exposuremin(self):
        """
        Return the minimum exposure time supported by StartExposure.
        """

        return self.get("exposuremin")

    def exposureresolution(self):
        """
        Return the smallest increment in exposure time supported by StartExposure.
        """

        return self.get("exposureresolution")

    def fastreadout(self, FastReadout=None):
        """
        Set or return whether Fast Readout Mode is enabled.
        :param
            FastReadout (bool): True to enable fast readout mode.

        :return:
            Whether Fast Readout Mode is enabled.
        """

        if FastReadout is None:
            return self.get("fastreadout")

        self.put("fastreadout", FastReadout=FastReadout)

    def fullwellcapacity(self):
        """
        Report the full well capacity of the camera.
        Report the full well capacity of the camera in electrons, at the current
        camera settings (binning, SetupDialog settings, etc.).
        :return:
            Full well capacity of the camera.
        """

        return self.get("fullwellcapacity")

    def gain(self, Gain=None):
        """
        Set or return an index into the Gains array.
        :param
            Gain (int): Index of the current camera gain in the Gains string array.

        :return:
            Index into the Gains array for the selected camera gain.
        """

        if Gain is None:
            return self.get("gain")

        self.put("gain", Gain=Gain)

    def gainmax(self):
        """
        Maximum value of Gain.
        """

        return self.get("gainmax")

    def gainmin(self):
        """
        Minimum value of Gain.
        """

        return self.get("gainmin")

    def gains(self):
        """
        Gains supported by the camera.
        """

        return self.get("gains")

    def hasshutter(self):
        """
        Indicate whether the camera has a mechanical shutter.
        """

        return self.get("hasshutter")

    def heatsinktemperature(self):
        """
        Return the current heat sink temperature.
        :return:
            Current heat sink temperature (called "ambient temperature" by some
            manufacturers) in degrees Celsius.
        """

        return self.get("heatsinktemperature")

    def imagearray(self):
        """
        Return an array of integers containing the exposure pixel values.
        Return an array of 32bit integers containing the pixel values from the last
        exposure. This call can return either a 2 dimension (monochrome images) or 3
        dimension (colour or multi-plane images) array of size NumX * NumY or NumX *
        NumY * NumPlanes. Where applicable, the size of NumPlanes has to be determined
        by inspection of the returned Array. Since 32bit integers are always returned
        by this call, the returned JSON Type value (0 = Unknown, 1 = short(16bit),
        2 = int(32bit), 3 = Double) is always 2. The number of planes is given in the
        returned Rank value. When de-serialising to an object it helps enormously to
        know the array Rank beforehand so that the correct data class can be used. This
        can be achieved through a regular expression or by direct parsing of the
        returned JSON string to extract the Type and Rank values before de-serialising.
        :return:
            Array of integers containing the exposure pixel values.

        """

        return self.get("imagearray")

    def imagearrayvariant(self):
        """
        Return an array of integers containing the exposure pixel values.
        Return an array of 32bit integers containing the pixel values from the last
        exposure. This call can return either a 2 dimension (monochrome images) or 3
        dimension (colour or multi-plane images) array of size NumX * NumY or NumX *
        NumY * NumPlanes. Where applicable, the size of NumPlanes has to be determined
        by inspection of the returned Array. Since 32bit integers are always returned
        by this call, the returned JSON Type value (0 = Unknown, 1 = short(16bit),
        2 = int(32bit), 3 = Double) is always 2. The number of planes is given in the
        returned Rank value. When de-serialising to an object it helps enormously to
        know the array Rank beforehand so that the correct data class can be used. This
        can be achieved through a regular expression or by direct parsing of the
        returned JSON string to extract the Type and Rank values before de-serialising.
        :return:
            Array of integers containing the exposure pixel values.

        """

        return self.get("imagearrayvariant")

    def imageready(self):
        """
        Indicate that an image is ready to be downloaded.
        """

        return self.get("imageready")

    def ispulseguiding(self):
        """
        Indicatee that the camera is pulse guideing.
        """

        return self.get("ispulseguiding")

    def lastexposureduration(self):
        """
        Report the actual exposure duration in seconds (i.e. shutter open time).
        """

        return self.get("lastexposureduration")

    def lastexposurestarttime(self):
        """
        Start time of the last exposure in FITS standard format.

        Reports the actual exposure start in the FITS-standard
        CCYY-MM-DDThh:mm:ss[.sss...] format.
        :return:
            Start time of the last exposure in FITS standard format.
        """

        return self.get("lastexposurestarttime")

    def maxadu(self):
        """
        Camera's maximum ADU value.
        """

        return self.get("maxadu")

    def maxbinx(self):
        """
        Maximum binning for the camera X axis.
        """

        return self.get("maxbinx")

    def maxbiny(self):
        """
        Maximum binning for the camera Y axis.
        """

        return self.get("maxbiny")

    def numx(self, NumX=None):
        """
        Set or return the current subframe width.

        :param
            NumX (int): Subframe width, if binning is active, value is in binned
                pixels.
        :return:
            Current subframe width.
        """

        if NumX is None:
            return self.get("numx")

        self.put("numx", NumX=NumX)

    def numy(self, NumY=None):
        """
        Set or return the current subframe height.

        :param
            NumY (int): Subframe height, if binning is active, value is in binned
                pixels.
        :return:
            Current subframe height.
        """

        if NumY is None:
            return self.get("numy")

        self.put("numy", NumY=NumY)

    def percentcompleted(self):
        """
        Indicate percentage completeness of the current operation.
        :return:
            If valid, returns an integer between 0 and 100, where 0 indicates 0%
            progress (function just started) and 100 indicates 100% progress (i.e.
            completion).

        """

        return self.get("percentcompleted")

    def pixelsizex(self):
        """
        Width of CCD chip pixels (microns).
        """

        return self.get("pixelsizex")

    def pixelsizey(self):
        """
        Height of CCD chip pixels (microns).
        """

        return self.get("pixelsizey")

    def readoutmode(self, ReadoutMode=None):
        """
        Indicate the camera's readout mode as an index into the array ReadoutModes.
        """

        if ReadoutMode is None:
            return self.get("readoutmode")

        self.put("readoutmode", ReadoutMode=ReadoutMode)

    def readoutmodes(self):
        """
        List of available readout modes.
        """

        return self.get("readoutmodes")

    def sensorname(self):
        """
        Name of the sensor used within the camera.
        """

        return self.get("sensorname")

    def sensortype(self):
        """
        Type of information returned by the the camera sensor (monochrome or colour).

        Notes:
            0 = Monochrome, 1 = Colour not requiring Bayer decoding, 2 = RGGB Bayer
            encoding, 3 = CMYG Bayer encoding, 4 = CMYG2 Bayer encoding, 5 = LRGB
            TRUESENSE Bayer encoding.

        :return:
            Value indicating whether the sensor is monochrome, or what Bayer matrix it
            encodes.
        """

        return self.get("sensortype")

    def setccdtemperature(self, SetCCDTemperature=None):
        """
        Set or return the camera's cooler setpoint (degrees Celsius).
        :param
            SetCCDTemperature (float): 	Temperature set point (degrees Celsius).

        :return:
            Camera's cooler setpoint (degrees Celsius).
        """

        if SetCCDTemperature is None:
            return self.get("setccdtemperature")

        self.put("setccdtemperature", SetCCDTemperature=SetCCDTemperature)

    def startx(self, StartX=None):
        """
        Set or return the current subframe X axis start position.
        :param
            StartX (int): The subframe X axis start position in binned pixels.

        :return:
            Sets the subframe start position for the X axis (0 based) and returns the
            current value. If binning is active, value is in binned pixels.
        """

        if StartX is None:
            return self.get("startx")

        self.put("startx", StartX=StartX)

    def starty(self, StartY=None):
        """
        Set or return the current subframe Y axis start position.
        :param
            StartY (int): The subframe Y axis start position in binned pixels.

        :return:
            Sets the subframe start position for the Y axis (0 based) and returns the
            current value. If binning is active, value is in binned pixels.
        """

        if StartY is None:
            return self.get("starty")

        self.put("starty", StartY=StartY)

    def abortexposure(self):
        """
        Abort the current exposure, if any, and returns the camera to Idle state.
        """

        self.put("abortexposure")

    def pulseguide(self, Direction: int, Duration: int):
        """
        Pulse guide in the specified direction for the specified time.

        :param
            Direction (int): Direction of movement (0 = North, 1 = South, 2 = East,
                3 = West).
            Duration (int): Duration of movement in milli-seconds.
        """

        self.put("pulseguide", Direction=Direction, Duration=Duration)

    def startexposure(self, Duration=0, Light=True):
        """
        Start an exposure.

        Notes:
            Use ImageReady to check when the exposure is complete.

        :param
            Duration (float): Duration of exposure in seconds.
            Light (bool): True if light frame, false if dark frame.
        """

        self.put("startexposure", Duration=Duration, Light=Light)

    def stopexposure(self):
        """
        Stop the current exposure, if any.

        Notes:
            If an exposure is in progress, the readout process is initiated. Ignored if
            readout is already in process.
        """

        self.put("stopexposure")


class FilterWheel(AlpacaBase):
    """
    Filter wheel specific methods.
    """

    def focusoffsets(self):
        """
        Filter focus offsets.
        :return:
            An integer array of filter focus offsets.

        """
        return self.get("focusoffsets")

    def names(self):
        """
        Filter wheel filter names.
        :return:
            Names of the filters.
        """

        return self.get("names")

    def position(self, Position=None):
        """
        Set or return the filter wheel position.
        :param
            Position (int): Number of the filter wheel position to select.
        :return:
            Returns the current filter wheel position.
        """

        if Position is None:
            return self.get("position")

        self.put("position", Position=Position)


class Telescope(AlpacaBase):
    """
    Telescope specific methods.
    """

    def alignmentmode(self):
        """
        Return the current mount alignment mode.
        :return:
            Alignment mode of the mount (Alt/Az, Polar, German Polar).

        """

        return self.get("alignmentmode")

    def altitude(self):
        """
        Return the mount's Altitude above the horizon.
        :return:
            Altitude of the telescope's current position (degrees, positive up).
        """

        return self.get("altitude")

    def aperturearea(self):
        """
        Return the telescope's aperture.
        :return:
            Area of the telescope's aperture (square meters).
        """

        return self.get("aperturearea")

    def aperturediameter(self):
        """
        Return the telescope's effective aperture.
        :return:
            Telescope's effective aperture diameter (meters).
        """

        value = self.get("aperturediameter")
        if value:
            return value * 1000

        else:
            return None

    def athome(self):
        """
        Indicate whether the mount is at the home position.
        :return:
            True if the mount is stopped in the Home position. Must be False if the
            telescope does not support homing.

        """

        return self.get("athome")

    def atpark(self):
        """
        Indicate whether the telescope is at the park position.
        :return:
            True if the telescope has been put into the parked state by the set park()
            method. Set False by calling the unpark() method.
        """

        return self.get("atpark")

    def azimuth(self):
        """
        Return the telescope's aperture.

        Return:
            Azimuth of the telescope's current position (degrees, North-referenced,
            positive East/clockwise).
        """

        return self.get("azimuth")

    def canfindhome(self):
        """
        Indicate whether the mount can find the home position.

        :return:
            True if this telescope is capable of programmed finding its home position.
        """

        return self.get("canfindhome")

    def canpark(self):
        """
        Indicate whether the telescope can be parked.
        :return:
            True if this telescope is capable of programmed parking.
        """

        return self.get("canpark")

    def canpulseguide(self):
        """
        Indicate whether the telescope can be pulse guided.
        :return:
            True if this telescope is capable of software-pulsed guiding (via the
            pulseguide(int, int) method).
        """

        return self.get("canpulseguide")

    def cansetdeclinationrate(self):
        """
        Indicate whether the DeclinationRate property can be changed.
        :return:
            True if the DeclinationRate property can be changed to provide offset
            tracking in the declination axis.
        """

        return self.get("cansetdeclinationrate")

    def cansetguiderates(self):
        """
        Indicate whether the DeclinationRate property can be changed.
        :return:
            True if the guide rate properties used for pulseguide(int, int) can ba
            adjusted.
        """

        return self.get("cansetguiderates")

    def cansetpark(self):
        """
        Indicate whether the telescope park position can be set.
        :return:
            True if this telescope is capable of programmed setting of its park position
            (setpark() method).
        """

        return self.get("cansetpark")

    def cansetpierside(self):
        """
        Indicate whether the telescope SideOfPier can be set.
        :return:
            True if the SideOfPier property can be set, meaning that the mount can be
            forced to flip.
        """

        return self.get("cansetpierside")

    def cansetrightascensionrate(self):
        """
        Indicate whether the RightAscensionRate property can be changed.
        :return:
            True if the RightAscensionRate property can be changed to provide offset
            tracking in the right ascension axis.
        """

        return self.get("cansetrightascensionrate")

    def cansettracking(self):
        """
        Indicate whether the Tracking property can be changed.
        :return:
            True if the Tracking property can be changed, turning telescope sidereal
            tracking on and off.
        """

        return self.get("cansettracking")

    def canslew(self):
        """
        Indicate whether the telescope can slew synchronously.
        :return:
            True if this telescope is capable of programmed slewing (synchronous or
            asynchronous) to equatorial coordinates.
        """

        return self.get("canslew")

    def canslewaltaz(self):
        """
        Indicate whether the telescope can slew synchronously to AltAz coordinates.
        :return:
            True if this telescope is capable of programmed slewing (synchronous or
            asynchronous) to local horizontal coordinates.
        """

        return self.get("canslewaltaz")

    def canslewaltazasync(self):
        """
        Indicate whether the telescope can slew asynchronusly to AltAz coordinates.
        :return:
            True if this telescope is capable of programmed asynchronus slewing
            (synchronous or asynchronous) to local horizontal coordinates.
        """

        return self.get("canslewaltazasync")

    def cansync(self):
        """
        Indicate whether the telescope can sync to equatorial coordinates.
        :return:
            True if this telescope is capable of programmed synching to equatorial
            coordinates.
        """

        return self.get("cansync")

    def cansyncaltaz(self):
        """
        Indicate whether the telescope can sync to local horizontal coordinates.
        :return:
            True if this telescope is capable of programmed synching to local horizontal
            coordinates.
        """

        return self.get("cansyncaltaz")

    def declination(self):
        """
        Return the telescope's declination.
        Notes:
            Reading the property will raise an error if the value is unavailable.
        :return:
            The declination (degrees) of the telescope's current equatorial coordinates,
            in the coordinate system given by the EquatorialSystem property.
        """

        return self.get("declination")

    def declinationrate(self, DeclinationRate=None):
        """
        Set or return the telescope's declination tracking rate.
        :param
            DeclinationRate (float): Declination tracking rate (arcseconds per second).

        :return:
            The declination tracking rate (arcseconds per second) if DeclinationRate is
            not set.
        """

        if DeclinationRate is None:
            return self.get("declinationrate")

        self.put("declinationrate", DeclinationRate=DeclinationRate)

    def doesrefraction(self, DoesRefraction=None):
        """
        Indicate or determine if atmospheric refraction is applied to coordinates.
        :param
            DoesRefraction (bool): Set True to make the telescope or driver apply
                atmospheric refraction to coordinates.

        :return:
            True if the telescope or driver applies atmospheric refraction to
            coordinates.
        """

        if DoesRefraction is None:
            return self.get("doesrefraction")

        self.put("doesrefraction", DoesRefraction=DoesRefraction)

    def equatorialsystem(self):
        """
        Return the current equatorial coordinate system used by this telescope.
        :return:
            Current equatorial coordinate system used by this telescope
            (e.g. Topocentric or J2000).
        """

        return self.get("equatorialsystem")

    def focallength(self):
        """
        Return the telescope's focal length in meters.
        :return:
            The telescope's focal length in meters.
        """

        value = self.get("focallength")
        if value:
            return value * 1000

        else:
            return None

    def guideratedeclination(self, GuideRateDeclination=None):
        """
        Set or return the current Declination rate offset for telescope guiding.
        :param
            GuideRateDeclination (float): Declination movement rate offset
                (degrees/sec).
        :return:
            Current declination rate offset for telescope guiding if not set.
        """

        if GuideRateDeclination is None:
            return self.get("guideratedeclination")

        self.put("guideratedeclination", GuideRateDeclination=GuideRateDeclination)

    def guideraterightascension(self, GuideRateRightAscension=None):
        """
        Set or return the current RightAscension rate offset for telescope guiding.
        :param
            GuideRateRightAscension (float): RightAscension movement rate offset
                (degrees/sec).
        :return:
            Current right ascension rate offset for telescope guiding if not set.
        """

        if GuideRateRightAscension is None:
            return self.get("guideraterightascension")

        self.put("guideraterightascension", GuideRateRightAscension=GuideRateRightAscension)

    def ispulseguiding(self):
        """
        Indicate whether the telescope is currently executing a PulseGuide command.
        :return:
            True if a pulseguide(int, int) command is in progress, False otherwise.

        """

        return self.get("ispulseguiding")

    def rightascension(self):
        """
        Return the telescope's right ascension coordinate.
        :return:
            The right ascension (hours) of the telescope's current equatorial
            coordinates, in the coordinate system given by the EquatorialSystem
            property.
        """

        return self.get("rightascension")

    def rightascensionrate(self, RightAscensionRate=None):
        """
        Set or return the telescope's right ascension tracking rate.
        :param
            RightAscensionRate (float): Right ascension tracking rate (arcseconds per
                second).
        :return:
            Telescope's right ascension tracking rate if not set.
        """

        if RightAscensionRate is None:
            return self.get("rightascensionrate")

        self.put("rightascensionrate", RightAscensionRate=RightAscensionRate)

    def sideofpier(self, SideOfPier=None):
        """
        Set or return the mount's pointing state.
        :param
            SideOfPier (int): New pointing state. 0 = pierEast, 1 = pierWest

        :return:
            Side of pier if not set.
        """

        if SideOfPier is None:
            return self.get("sideofpier")

        self.put("sideofpier", SideOfPier=SideOfPier)

    def siderealtime(self):
        """
        Return the local apparent sidereal time.
        :return:
            The local apparent sidereal time from the telescope's internal clock (hours,
            sidereal).
        """

        return self.get("siderealtime")

    def siteelevation(self, SiteElevation=None):
        """
        Set or return the observing site's elevation above mean sea level.
        :param
            SiteElevation (float): Elevation above mean sea level (metres).

        :return:
            Elevation above mean sea level (metres) of the site at which the telescope
            is located if not set.
        """

        if SiteElevation is None:
            return self.get("siteelevation")

        self.put("siteelevation", SiteElevation=SiteElevation)

    def sitelatitude(self, SiteLatitude=None):
        """
        Set or return the observing site's latitude.
        :param
            SiteLatitude (float): Site latitude (degrees).

        :return:
            Geodetic(map) latitude (degrees, positive North, WGS84) of the site at which
            the telescope is located if not set.
        """

        if SiteLatitude is None:
            return self.get("sitelatitude")

        self.put("sitelatitude", SiteLatitude=SiteLatitude)

    def sitelongitude(self, SiteLongitude=None):
        """
        Set or return the observing site's longitude.
        :param
            SiteLongitude (float): Site longitude (degrees, positive East, WGS84)

        :return:
            Longitude (degrees, positive East, WGS84) of the site at which the telescope
            is located.
        """

        if SiteLongitude is None:
            return self.get("sitelongitude")

        self.put("sitelongitude", SiteLongitude=SiteLongitude)

    def slewing(self):
        """
        Indicate whether the telescope is currently slewing.
        :return:
            True if telescope is currently moving in response to one of the Slew methods
            or the moveaxis(int, float) method, False at all other times.
        """

        return self.get("slewing")

    def slewsettletime(self, SlewSettleTime=None):
        """
        Set or return the post-slew settling time.
        :param
            SlewSettleTime (int): Settling time (integer sec.).
        :return:
            Returns the post-slew settling time (sec.) if not set.
        """

        if SlewSettleTime is None:
            return self.get("slewsettletime")

        self.put("slewsettletime", SlewSettleTime=SlewSettleTime)

    def targetdeclination(self, TargetDeclination=None):
        """
        Set or return the target declination of a slew or sync.
        :param
            TargetDeclination (float): Target declination(degrees)

        :return:
            Declination (degrees, positive North) for the target of an equatorial slew
            or sync operation.
        """

        if TargetDeclination is None:
            return self.get("targetdeclination")

        self.put("targetdeclination", TargetDeclination=TargetDeclination)

    def targetrightascension(self, TargetRightAscension=None):
        """
        Set or return the current target right ascension.
        :param
            TargetRightAscension (float): Target right ascension (hours).

        :return:
            Right ascension (hours) for the target of an equatorial slew or sync
            operation.
        """

        if TargetRightAscension is None:
            return self.get("targetrightascension")

        self.put("targetrightascension", TargetRightAscension=TargetRightAscension)

    def tracking(self, Tracking=None):
        """
        Enable, disable, or indicate whether the telescope is tracking.
        :param
            Tracking (bool): Tracking enabled / disabled.

        :return:
            State of the telescope's sidereal tracking drive.
        """

        if Tracking is None:
            return self.get("tracking")

        self.put("tracking", Tracking=Tracking)

    def trackingrate(self, TrackingRate=None):
        """
        Set or return the current tracking rate.
        :param
            TrackingRate (int): New tracking rate. 0 = driveSidereal, 1 = driveLunar,
                2 = driveSolar, 3 = driveKing.

        :return:
            Current tracking rate of the telescope's sidereal drive if not set.
        """

        if TrackingRate is None:
            return self.get("trackingrate")

        self.put("trackingrate", TrackingRate=TrackingRate)

    def trackingrates(self):
        """
        Return a collection of supported DriveRates values.
        :return:
            List of supported DriveRates values that describe the permissible values of
            the TrackingRate property for this telescope type.
        """

        return self.get("trackingrates")

    def utcdate(self, UTCDate=None):
        """
        Set or return the UTC date/time of the telescope's internal clock.
        :param
            UTCDate: UTC date/time as an str or datetime.

        :return:
            datetime of the UTC date/time if not set.
        """

        if UTCDate is None:
            return parser.parse(self.get("utcdate"), fuzzy=True)

        if type(UTCDate) is str:
            data = UTCDate

        elif type(UTCDate) is datetime.datetime:
            data = UTCDate.isoformat()

        else:
            self.log.warning(f'type error in: [{UTCDate}]')
            return None

        self.put("utcdate", UTCDate=data)

    def abortslew(self):
        """
        Immediately stops a slew in progress.
        """

        self.put("abortslew")

    def axisrates(self, Axis):
        """
        Return rates at which the telescope may be moved about the specified axis.
        :return:
            The rates at which the telescope may be moved about the specified axis by
            the moveaxis(int, float) method.
        """

        return self.get("axisrates", Axis=Axis)

    def canmoveaxis(self, Axis):
        """
        Indicate whether the telescope can move the requested axis.
        :return:
            True if this telescope can move the requested axis.
        """

        return self.get("canmoveaxis", Axis=Axis)

    def destinationsideofpier(self, RightAscension, Declination):
        """
        Predict the pointing state after a German equatorial mount slews to given coordinates.
        :param
            RightAscension (float): Right Ascension coordinate (0.0 to 23.99999999
                hours).
            Declination (float): Declination coordinate (-90.0 to +90.0 degrees).
        :return:
            Pointing state that a German equatorial mount will be in if it slews to the
            given coordinates. The return value will be one of - 0 = pierEast,
            1 = pierWest, -1 = pierUnknown.
        """

        return self.get(
            "destinationsideofpier",
            RightAscension=RightAscension,
            Declination=Declination,
        )

    def findhome(self):
        """
        Move the mount to the "home" position.
        """

        self.put("findhome")

    def moveaxis(self, Axis, Rate):
        """
        Move a telescope axis at the given rate.
        :param
            Axis (int): The axis about which rate information is desired.
                0 = axisPrimary, 1 = axisSecondary, 2 = axisTertiary.
            Rate (float): The rate of motion (deg/sec) about the specified axis
        """

        self.put("moveaxis", Axis=Axis, Rate=Rate)

    def park(self):
        """
        Park the mount.
        """

        self.put("park")

    def pulseguide(self, Direction, Duration):
        """
        Move the scope in the given direction for the given time.
        Notes:
            0 = guideNorth, 1 = guideSouth, 2 = guideEast, 3 = guideWest.
        :param
            Direction (int): Direction in which the guide-rate motion is to be made.
            Duration (int): Duration of the guide-rate motion (milliseconds).
        """

        self.put("pulseguide", Direction=Direction, Duration=Duration)

    def setpark(self):
        """
        Set the telescope's park position.
        """

        self.put("setpark")

    def slewtoaltaz(self, Azimuth, Altitude):
        """
        Slew synchronously to the given local horizontal coordinates.
        :param
            Azimuth (float): Azimuth coordinate (degrees, North-referenced, positive
                East/clockwise).
            Altitude (float): Altitude coordinate (degrees, positive up).
        """

        self.put("slewtoaltaz", Azimuth=Azimuth, Altitude=Altitude)

    def slewtoaltazasync(self, Azimuth, Altitude):
        """
        Slew asynchronously to the given local horizontal coordinates.
        :param
            Azimuth (float): Azimuth coordinate (degrees, North-referenced, positive
                East/clockwise).
            Altitude (float): Altitude coordinate (degrees, positive up).
        """

        self.put("slewtoaltazasync", Azimuth=Azimuth, Altitude=Altitude)

    def slewtocoordinates(self, RightAscension, Declination):
        """
        Slew synchronously to the given equatorial coordinates.
        :param
            RightAscension (float): Right Ascension coordinate (hours).
            Declination (float): Declination coordinate (degrees).
        """

        self.put("slewtocoordinates", RightAscension=RightAscension, Declination=Declination)

    def slewtocoordinatesasync(self, RightAscension, Declination):
        """
        Slew asynchronously to the given equatorial coordinates.
        :param
            RightAscension (float): Right Ascension coordinate (hours).
            Declination (float): Declination coordinate (degrees).
        """

        self.put(
            "slewtocoordinatesasync",
            RightAscension=RightAscension,
            Declination=Declination,
        )

    def slewtotarget(self):
        """
        Slew synchronously to the TargetRightAscension and TargetDeclination coordinates.
        """

        self.put("slewtotarget")

    def slewtotargetasync(self):
        """
        Asynchronously slew to the TargetRightAscension and TargetDeclination coordinates.
        """

        self.put("slewtotargetasync")

    def synctoaltaz(self, Azimuth, Altitude):
        """
        Sync to the given local horizontal coordinates.
        :param
            Azimuth (float): Azimuth coordinate (degrees, North-referenced, positive
                East/clockwise).
            Altitude (float): Altitude coordinate (degrees, positive up).
        """

        self.put("synctoaltaz", Azimuth=Azimuth, Altitude=Altitude)

    def synctocoordinates(self, RightAscension, Declination):
        """
        Sync to the given equatorial coordinates.
        :param
            RightAscension (float): Right Ascension coordinate (hours).
            Declination (float): Declination coordinate (degrees).
        """

        self.put(
            "synctocoordinates", RightAscension=RightAscension, Declination=Declination
        )

    def synctotarget(self):
        """
        Sync to the TargetRightAscension and TargetDeclination coordinates.
        """

        self.put("synctotarget")

    def unpark(self):
        """
        Unpark the mount.
        """

        self.put("unpark")


class ObservingConditions(AlpacaBase):
    """
    ObservingConditions  specific methods.
    """

    def averageperiod(self, AveragePeriod=None):
        if AveragePeriod is None:
            return self.get("averageperiod")

        self.put("averageperiod", AveragePeriod=AveragePeriod)

    def cloudcover(self):
        return self.get("cloudcover")

    def dewpoint(self):
        return self.get("dewpoint")

    def humidity(self):
        return self.get("humidity")

    def pressure(self):
        return self.get("pressure")

    def rainrate(self):
        return self.get("rainrate")

    def skybrightness(self):
        return self.get("skybrightness")

    def skyquality(self):
        return self.get("skyquality")

    def skytemperature(self):
        return self.get("skytemperature")

    def starfwhm(self):
        return self.get("starfwhm")

    def temperature(self):
        return self.get("temperature")

    def winddirection(self):
        return self.get("winddirection")

    def windgust(self):
        return self.get("windgust")

    def windspeed(self):
        return self.get("windspeed")

    def refresh(self):
        self.put("refresh")

    def sensordescription(self):
        return self.get("sensordescription")

    def timesincelastupdate(self):
        return self.get("timesincelastupdate")


class Focuser(AlpacaBase):
    """
    Focuser  specific methods.
    """

    def absolut(self):
        return self.get("absolut")

    def ismoving(self):
        return self.get("ismoving")

    def maxincrement(self):
        return self.get("maxincrement")

    def maxstep(self):
        return self.get("maxstep")

    def position(self):
        return self.get("position")

    def stepsize(self):
        return self.get("stepsize")

    def tempcomp(self, TempComp=None):
        if TempComp is None:
            return self.get("tempcomp")

        self.put("tempcomp", TempComp=TempComp)

    def tempcompavailable(self):
        return self.get("tempcompavailable")

    def temperature(self):
        return self.get("temperature")

    def halt(self):
        self.put("halt")

    def move(self, Position=None):
        if Position is None:
            return self.get("move")

        self.put("move", Position=int(Position))


class Covercalibrator(AlpacaBase):
    """
    Covercalibrator  specific methods.
    """

    def brightness(self):
        return self.get("brightness")

    def calibratoroff(self):
        self.put("calibratoroff")

    def calibratoron(self, Brightness=None):
        if Brightness:
            self.put("calibratoron", Brightness=Brightness)

    def calibratorstate(self):
        return self.get("calibratorstate")

    def closecover(self):
        self.put("closecover")

    def opencover(self):
        self.put("opencover")

    def haltcover(self):
        self.put("haltcover")

    def coverstate(self):
        return self.get("coverstate")

    def maxbrightness(self):
        return self.get("maxbrightness")



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
# Licence APL2.0
#
###########################################################
# standard libraries
import logging

# external packages
import PyQt5.QtCore
import numpy as np
import requests

# local imports
from base.tpool import Worker


class WeatherSignals(PyQt5.QtCore.QObject):
    """
    The WeatherSignals class offers a list of signals to be used and instantiated by
    the Mount class to get signals for triggers for finished tasks to
    enable a gui to update their values transferred to the caller back.

    This has to be done in a separate class as the signals have to be subclassed from
    QObject and the Mount class itself is subclassed from object
    """

    __all__ = ['WeatherSignals']

    dataReceived = PyQt5.QtCore.pyqtSignal(object)
    connected = PyQt5.QtCore.pyqtSignal(object)

    serverConnected = PyQt5.QtCore.pyqtSignal()
    serverDisconnected = PyQt5.QtCore.pyqtSignal(object)
    deviceConnected = PyQt5.QtCore.pyqtSignal(str)
    deviceDisconnected = PyQt5.QtCore.pyqtSignal(str)


class OnlineWeather(PyQt5.QtCore.QObject):
    """
    the class Skymeter inherits all information and handling of the Skymeter device

        >>> weather = OnlineWeather(app)
    """

    __all__ = ['OnlineWeather',
               ]

    log = logging.getLogger(__name__)

    def __init__(self,
                 app=None,
                 ):
        super().__init__()

        self.app = app
        self.threadPool = app.threadPool
        self.signals = WeatherSignals()
        self.location = app.mount.obsSite.location

        # minimum set for driver package built in
        self.framework = ''
        self.run = {
            'onlineWeather': self
        }
        self.deviceName = ''

        self.data = {}
        self.defaultConfig = {
            'framework': '',
            'frameworks': {
                'onlineWeather': {
                    'deviceName': 'OnlineWeather',
                    'apiKey': '',
                    'hostaddress': 'api.openweathermap.org',
                }
            }
        }

        self.running = False
        self.hostaddress = ''
        self.apiKey = ''
        self._online = False

        self.app.update10s.connect(self.updateOpenWeatherMapData)

    @property
    def online(self):
        return self._online

    @online.setter
    def online(self, value):
        self._online = value
        if value:
            self.updateOpenWeatherMapData()

    def startCommunication(self, loadConfig=False):
        """
        startCommunication adds a device on the watch list of the server.

        :param loadConfig:
        :return: success of reconnecting to server
        """

        if not self.apiKey:
            return False

        self.running = True
        self.signals.deviceConnected.emit('OnlineWeather')
        self.updateOpenWeatherMapData()

        return True

    def stopCommunication(self):
        """
        stopCommunication adds a device on the watch list of the server.

        :return: success of reconnecting to server
        """

        self.running = False
        self.data.clear()
        self.signals.deviceDisconnected.emit('OnlineWeather')

        return True

    @staticmethod
    def getDewPoint(tempAir, relativeHumidity):
        """
        Compute the dew point in degrees Celsius

        :param tempAir: current ambient temperature in degrees Celsius
        :param relativeHumidity: relative humidity in %
        :return: the dew point in degrees Celsius
        """

        if tempAir < -40 or tempAir > 80:
            return 0

        if relativeHumidity < 0 or relativeHumidity > 100:
            return 0

        A = 17.27
        B = 237.7
        alpha = ((A * tempAir) / (B + tempAir)) + np.log(relativeHumidity / 100.0)
        dewPoint = (B * alpha) / (A - alpha)

        return dewPoint

    def updateOpenWeatherMapDataWorker(self, data=None):
        """
        updateOpenWeatherMapDataWorker takes the returned data from a web fetch and puts
        the data in a dict

        :param data:

        :return: success
        """

        if data is None:
            self.signals.dataReceived.emit(None)
            self.signals.connected.emit(False)
            return False

        if 'list' not in data:
            self.signals.dataReceived.emit(None)
            self.signals.connected.emit(False)
            return False

        if len(data['list']) == 0:
            self.signals.dataReceived.emit(None)
            self.signals.connected.emit(False)
            return False

        val = data['list'][0]
        self.log.trace(f'onlineWeatherData:[{val}]')

        if 'main' in val:
            self.data['temperature'] = val['main']['temp'] - 273.15
            self.data['pressure'] = val['main']['grnd_level']
            self.data['humidity'] = val['main']['humidity']
            self.data['dewPoint'] = self.getDewPoint(self.data['temperature'],
                                                     self.data['humidity'])
        if 'clouds' in val:
            self.data['cloudCover'] = val['clouds']['all']

        if 'wind' in val:
            self.data['windSpeed'] = val['wind']['speed']
            self.data['windDir'] = val['wind']['deg']

        if 'rain' in val:
            self.data['rain'] = val['rain']['3h']

        self.signals.dataReceived.emit(self.data)
        self.signals.connected.emit(True)

        return True

    def getOpenWeatherMapDataWorker(self, url=''):
        """
        getOpenWeatherMapDataWorker fetches a given url and does the error handling.

        :param url:
        :return: data
        """

        if not url:
            return None

        try:
            data = requests.get(url, timeout=30)

        except TimeoutError:
            self.log.warning(f'{url} not reachable')
            return None

        except Exception as e:
            self.log.critical(f'{url} general exception: {e}')
            return None

        if data.status_code != 200:
            self.log.warning(f'{url}: status nok')
            return None

        return data.json()

    def getOpenWeatherMapData(self, url=''):
        """
        getOpenWeatherMapData initiates the worker thread to get the web data fetched

        :param url:
        :return: true for test purpose
        """

        worker = Worker(self.getOpenWeatherMapDataWorker, url)
        worker.signals.result.connect(self.updateOpenWeatherMapDataWorker)
        self.threadPool.start(worker)

        return True

    def updateOpenWeatherMapData(self):
        """
        updateOpenWeatherMap downloads the actual OpenWeatherMap image and displays it in
        environment tab. it checks first if online is set, otherwise not download will take
        place. it will be updated every 10 minutes.

        :return: success
        """

        if not self.apiKey:
            self.signals.connected.emit(None)
            return False

        if not self.online:
            self.signals.connected.emit(False)
            self.signals.dataReceived.emit(None)
            return False

        if not self.running:
            self.signals.connected.emit(False)
            return False

        # prepare coordinates for website
        lat = self.location.latitude.degrees
        lon = self.location.longitude.degrees

        webSite = f'http://{self.hostaddress}/data/2.5/forecast'
        url = f'{webSite}?lat={lat:1.2f}&lon={lon:1.2f}&APPID={self.apiKey}'
        self.getOpenWeatherMapData(url=url)
        self.log.debug(f'{url}')

        return True
